Package: CytoPipelineGUI
Title: GUI's for visualization of flow cytometry data analysis
        pipelines
Version: 1.8.0
Authors@R: 
    c(person(given = "Philippe",
             family = "Hauchamps",
             role = c("aut", "cre"),
             email = "philippe.hauchamps@uclouvain.be",
             comment = c(ORCID = "0000-0003-2865-1852")),
      person(given = "Laurent", family = "Gatto",
             email = "laurent.gatto@uclouvain.be",
             role = "aut",
             comment = c(ORCID = "0000-0002-1520-2268")),
      person(given = "Dan",
             family = "Lin",
             role = "ctb",
             email = "dan.8.lin@gsk.com"))
Description: This package is the companion of the `CytoPipeline` package.
 It provides GUI's (shiny apps) for the visualization of flow cytometry data
 analysis pipelines that are run with `CytoPipeline`.
 Two shiny applications are provided, i.e.
 an interactive flow frame assessment and comparison tool and
 an interactive scale transformations visualization and adjustment tool.
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
BugReports: https://github.com/UCLouvain-CBIO/CytoPipelineGUI/issues
URL: https://uclouvain-cbio.github.io/CytoPipelineGUI
biocViews: FlowCytometry, Preprocessing, QualityControl, WorkflowStep,
        ImmunoOncology, Software, Visualization, GUI, ShinyApps
Collate: plots.R shiny-functions.R shiny-scaleTransform-module.R
        shiny.R CytoPipelineGUI-package.R
Depends: R (>= 4.4), CytoPipeline (>= 1.9.3)
Imports: shiny, plotly, ggplot2, flowCore
Suggests: testthat (>= 3.0.0), vdiffr, diffviewer, knitr, rmarkdown,
        BiocStyle, patchwork
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/CytoPipelineGUI
git_branch: RELEASE_3_22
git_last_commit: 9a3910e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:27:05 UTC; biocbuild
Author: Philippe Hauchamps [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2865-1852>),
  Laurent Gatto [aut] (ORCID: <https://orcid.org/0000-0002-1520-2268>),
  Dan Lin [ctb]
Maintainer: Philippe Hauchamps <philippe.hauchamps@uclouvain.be>

# Generated by roxygen2: do not edit by hand

export(CytoPipelineCheckApp)
export(ScaleTransformApp)
export(plotDiffFlowFrame)
export(plotScaleTransformedChannel)
export(plotSelectedFlowFrame)
export(plotSelectedWorkflow)
import(CytoPipeline)
import(shiny)

## Automation and visualization of flow cytometry data analysis pipelines

[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![R-CMD-check-bioc](https://github.com/UCLouvain-CBIO/CytoPipelineGUI/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/UCLouvain-CBIO/CytoPipelineGUI/actions?query=workflow%3AR-CMD-check-bioc) 
[![license](https://img.shields.io/badge/license-GPL3.0-blue)](https://opensource.org/licenses/GPL-3.0)
<!--- [![codecov.io](https://codecov.io/github/UCLouvain-CBIO/CytoPipelineGUI/coverage.svg?branch=main)](https://codecov.io/github/UCLouvain-CBIO/CytoPipelineGUI?branch=main) -->


### What is CytoPipelineGUI?

`CytoPipelineGUI` is the companion package of `CytoPipeline`, and is used for
interactive visualization. It implements two shiny applications :
- a shiny app for interactive comparison of flow frames that are the results 
of CytoProcessingSteps of the same or different CytoPipeline experiments.  
It is launched using the following statement: `CytoPipelineCheckApp()`  
- a shiny app for interactive visualization and manual adjustments of scale 
transformation objects. It is launched using the following statement: 
`ScaleTransformApp()` 

### License

The `CytoPipelineGUI` code is provided under [GPL license version 3.0 or 
higher](https://opensource.org/licenses/GPL-3.0). The documentation, 
including the manual pages and the vignettes, are distributed under a [CC BY-SA 
4.0 license](https://creativecommons.org/licenses/by-sa/4.0/).

### Citation

If you use `CytopipelineGUI` in your research, 
please use the following citation:

>Hauchamps P, Bayat B, Delandre S, Hamrouni M, Toussaint M, Temmerman S, 
> Lin D, Gatto L (2024).
> “CytoPipeline and CytoPipelineGUI: a Bioconductor R package suite 
> for building and visualizing automated pre-processing pipelines 
> for flow cytometry data.” _BMC Bioinformatics_, *25*(1), 80. 
> doi:10.1186/s12859-024-05691-z <https://doi.org/10.1186/s12859-024-05691-z>.

or run `citation("CytoPipelineGUI")` to get the bibtex entry.

# CytoPipelineGUI - Copyright (C) <2022-2025> 
# <Université catholique de Louvain (UCLouvain), Belgique>
#   
#   Description and complete License: see LICENSE file.
# 
# This program (CytoPipelineGUI) is free software: 
#   you can redistribute it and/or modify it under the terms of the GNU 
#   General Public License as published by the Free Software Foundation, 
#   either version 3 of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).

#' @title Plot a pipeline workflow from a CytoPipeline run
#'
#' @param experimentName the experiment name (representing a pipeline run) 
#' from which to extract the workflow
#' @param whichQueue "pre-processing" or "scale transform"
#' @param sampleFile in case 'whichQueue' is set to 'pre-processing, which
#' sample file to look at. This can be a number or a character.
#' - if whichQueue == "scale transform", the sampleFile is ignored
#' - if NULL and whichQueue == "pre-processing", the sampleFile is defaulted
#' to the first one belonging to the experiment
#' @param path the root path to look for the CytoPipeline experiment cache
#'
#' @return nothing, but displays the plot as a side effect
#' @export
#'
#' @examples
#' 
#' # run CytoPipeline object first
#' 
#' outputDir <- base::tempdir()
#' 
#' 
#' rawDataDir <-
#'     system.file("extdata", package = "CytoPipeline")
#' experimentName <- "OMIP021_PeacoQC"
#' sampleFiles <- 
#'     file.path(
#'         rawDataDir, 
#'         list.files(rawDataDir, pattern = "Donor"))
#' jsonDir <- system.file("extdata", package = "CytoPipeline")
#' jsonPath <- file.path(jsonDir, "pipelineParams.json")
#' 
#' pipL2 <- CytoPipeline(
#'     jsonPath,
#'     experimentName = experimentName,
#'     sampleFiles = sampleFiles)
#' 
#' suppressWarnings(execute(
#'     pipL2,
#'     rmCache = TRUE,
#'     path = outputDir))
#' 
#' plotSelectedWorkflow(
#'     experimentName = experimentName,
#'     whichQueue = "pre-processing",
#'     sampleFile = sampleFiles[1],
#'     path = outputDir)
#'     
#' plotSelectedWorkflow(
#'     experimentName = experimentName,
#'     whichQueue = "scale transform",
#'     sampleFile = NULL,
#'     path = outputDir)
#' 
#' 
plotSelectedWorkflow <- function(
    experimentName,
    whichQueue,
    sampleFile,
    path = path) {
    pipL <- buildCytoPipelineFromCache(
        experimentName,
        path = path)
    
    if (whichQueue == "scale transform") {
        plotCytoPipelineProcessingQueue(
                pipL,
                whichQueue = whichQueue,
                sampleFile = NULL,
                path = path,
                title = FALSE,
                box.size = 0.15,
                box.prop = 0.3)
    } else if (is.null(sampleFile) ||
               is.numeric(sampleFile)) {
        plotCytoPipelineProcessingQueue(
            pipL,
            whichQueue = whichQueue,
            sampleFile = sampleFile,
            path = path,
            title = FALSE,
            box.size = 0.15,
            box.prop = 0.3)
        
    } else if (is.character(sampleFile)){
        if (sampleFile %in% sampleFiles(pipL)){
            plotCytoPipelineProcessingQueue(
                pipL,
                whichQueue = whichQueue,
                sampleFile = sampleFile,
                path = path,
                title = FALSE,
                box.size = 0.15,
                box.prop = 0.3)
        } else {
            displayNames <- sampleDisplayNames(pipL)
            if (sampleFile %in% displayNames) {
                actualSampleFile <- sampleNameFromDisplayName(pipL, sampleFile)
                plotCytoPipelineProcessingQueue(
                    pipL,
                    whichQueue = whichQueue,
                    sampleFile = actualSampleFile,
                    path = path,
                    title = FALSE,
                    box.size = 0.15,
                    box.prop = 0.3)
            }
        } # else do nothing
    }
}

#' @title Plot a flow frame from a CytoPipeline run
#' @description Based on an experiment name, this function will gather the
#' required flowFrame from the CytoPipeline disk cache and display it using 
#' the user chosen 1D or 2D view.
#' @param experimentName the experiment name (representing a pipeline run) 
#' from which to extract the flow frame
#' @param whichQueue "pre-processing" or "scale transform"
#' @param sampleFile in case 'whichQueue' is set to 'pre-processing, which
#' sample file to look at. This can be a number or a character.
#' - if whichQueue == "scale transform", the sampleFile is ignored
#' - if NULL and whichQueue == "pre-processing", the sampleFile is defaulted
#' to the first one belonging to the experiment
#' @param flowFrameName the name of the object to fetch   
#' (as referenced in the pipeline workflow)
#' @param path the root path to look for the CytoPipeline experiment cache
#' @param xChannelLabel the label of the channel to be displayed on the x axis:
#' the conventional syntax is : `channelName` + " - " + `channelMarker`
#' @param yChannelLabel the label of the channel to be displayed on the y axis:
#' the conventional syntax is : `channelName` + " - " + `channelMarker`
#' @param useAllCells if TRUE, no subsampling will be done
#' @param nDisplayCells if useAllCells == FALSE, the number of subsampled cells
#' @param useFixedLinearRange if TRUE, all channels using a linear scale will
#' use a fixed range set by linearRange
#' @param linearRange set for all channels using a linear scale,
#' if useFixedLinearRange == TRUE
#' @param transfoListName if not set to " ", the transformation list 
#' (as an object name ending with "_obj", as referenced in the pipeline 
#' workflow) to be used for for display. 
#' @return a ggplot object
#' @export
#'
#' @examples
#' 
#' # run CytoPipeline object first
#' 
#' outputDir <- base::tempdir()
#' 
#' 
#' rawDataDir <-
#'     system.file("extdata", package = "CytoPipeline")
#' experimentName <- "OMIP021_PeacoQC"
#' sampleFiles <- 
#'     file.path(
#'         rawDataDir, 
#'         list.files(rawDataDir, pattern = "Donor"))
#' jsonDir <- system.file("extdata", package = "CytoPipeline")
#' jsonPath <- file.path(jsonDir, "pipelineParams.json")
#' 
#' pipL2 <- CytoPipeline(
#'     jsonPath,
#'     experimentName = experimentName,
#'     sampleFiles = sampleFiles)
#' 
#' suppressWarnings(execute(
#'     pipL2,
#'     rmCache = TRUE,
#'     path = outputDir))
#' 
#' plotSelectedFlowFrame(
#'     experimentName = experimentName,
#'     whichQueue = "pre-processing",
#'     sampleFile = 1,
#'     flowFrameName = "remove_debris_obj",
#'     path = outputDir,
#'     xChannelLabel = "FSC-A : NA",
#'     yChannelLabel = "SSC-A : NA",
#'     useAllCells = TRUE,
#'     nDisplayCells = 0,
#'     useFixedLinearRange = TRUE,
#'     linearRange = c(-100, 262144))
#' 
#' plotSelectedFlowFrame(
#'     experimentName = experimentName,
#'     whichQueue = "pre-processing",
#'     sampleFile = 1,
#'     flowFrameName = "remove_debris_obj",
#'     path = outputDir,
#'     xChannelLabel = "FSC-A : NA",
#'     yChannelLabel = "SSC-A : NA",
#'     useAllCells = FALSE,
#'     nDisplayCells = 100,
#'     useFixedLinearRange = FALSE,
#'     linearRange = NULL)
#' 
#' plotSelectedFlowFrame(
#'     experimentName = experimentName,
#'     whichQueue = "pre-processing",
#'     sampleFile = 1,
#'     flowFrameName = "remove_debris_obj",
#'     path = outputDir,
#'     xChannelLabel = "Comp-670/30Violet-A : BV785 - CD3",
#'     yChannelLabel = "Comp-780/60Red-A : APCCy7 - CD4",
#'     useAllCells = TRUE,
#'     nDisplayCells = 0,
#'     useFixedLinearRange = FALSE,
#'     linearRange = NULL,
#'     transfoListName = "scale_transform_estimate_obj")
#' 
plotSelectedFlowFrame <- function(
        experimentName,
        whichQueue,
        sampleFile,
        flowFrameName,
        path,
        xChannelLabel,
        yChannelLabel,
        useAllCells,
        nDisplayCells,
        useFixedLinearRange,
        linearRange,
        transfoListName = " ") {
    if (xChannelLabel != " ") {
        message("displaying flow frame plot...")
        pipL <- buildCytoPipelineFromCache(
            experimentName,
            path = path)
        if (is.character(sampleFile)){
            if (!sampleFile %in% sampleFiles(pipL)){
                displayNames <- sampleDisplayNames(pipL)
                if (sampleFile %in% displayNames) {
                    sampleFile <- sampleNameFromDisplayName(pipL, sampleFile)
                } # else do nothing
            }
        }
        ff <- getCytoPipelineFlowFrame(
            pipL,
            whichQueue = whichQueue,
            sampleFile = sampleFile,
            objectName = flowFrameName,
            path = path)

        xChannel <- channelLabel2Name(xChannelLabel, ff)
        if (length(xChannel) == 0) return()

        fluoChannels <- flowCore::colnames(ff)[areFluoCols(ff)]

        transList <- NULL
        if (transfoListName != " ") {
            transList <-
                getCytoPipelineScaleTransform(
                    pipL,
                    whichQueue = "scale transform",
                    objectName = transfoListName,
                    path = path)
        }

        xScale <- "linear"
        if (xChannel %in% fluoChannels) {
            xScale <- "logicle"
        }

        yScale <- "linear"
        if (yChannelLabel == " ") {
            yChannel <- NULL
        } else {
            yChannel <- channelLabel2Name(yChannelLabel, ff)
            if (length(yChannel) == 0) return()
            if (yChannel %in% fluoChannels) {
                yScale <- "logicle"
            }
        }


        if (useFixedLinearRange) {
            xLinearRange <- linearRange
            yLinearRange <- linearRange
        } else {
            xLinearRange <- NULL
            yLinearRange <- NULL
        }

        if (useAllCells) {
            nDisplayCells <- Inf
        }

        p <- ggplotEvents(
            obj = ff,
            xChannel = xChannel,
            yChannel = yChannel,
            nDisplayCells = nDisplayCells,
            seed = 0,
            bins = 216,
            fill = "lightgreen",
            alpha = 0.2,
            xScale = xScale,
            yScale = yScale,
            xLinearRange = xLinearRange,
            yLinearRange = yLinearRange,
            transList = transList)


        theSubtitle <- paste0("nb of events: ", flowCore::nrow(ff))

        p <- p + ggplot2::labs(subtitle = theSubtitle)
        p

    }
}

#' @title Plot the difference plot between two flow frames 
#' from a CytoPipeline run
#' @description Based on an experiment name, this function will gather the
#' required flowFrames from the CytoPipeline disk cache and display 
#' a difference plot using the user chosen 1D or 2D view.
#' @param experimentNameFrom the experiment name (representing a pipeline run) 
#' from which to extract the flow frame ('from' situation)
#' @param experimentNameTo the experiment name (representing a pipeline run) 
#' from which to extract the flow frame ('to' situation)
#' @param whichQueueFrom "pre-processing" or "scale transform" 
#' ('from' situation)
#' @param whichQueueTo "pre-processing" or "scale transform" 
#' ('to' situation)
#' @param sampleFileFrom in case 'whichQueueFrom' is set to 'pre-processing, 
#' which sample file to look at for the 'from' situation. 
#' This can be a number or a character.
#' - if whichQueueFrom == "scale transform", the sampleFileFrom is ignored
#' - if NULL and whihQueueFrom == "pre-processing", the sampleFileFrom 
#' is defaulted to the first one belonging to the experiment
#' @param sampleFileTo same as sampleFileFrom, but for the 'to' situation
#' @param path the root path to look for the CytoPipeline experiment cache
#' @param flowFrameNameFrom for the 'from' situation, 
#' the name of the object to fetch (as referenced in the pipeline workflow)
#' @param flowFrameNameTo for the 'to' situation, 
#' the name of the object to fetch (as referenced in the pipeline workflow)
#' @param xChannelLabelFrom the label of the channel 
#' to be displayed on the x axis:
#' the conventional syntax is : `channelName` + " - " + `channelMarker`
#' @param xChannelLabelTo should be equal to xChannelLabelFrom
#' (otherwise no plot is returned but NULL)
#' @param yChannelLabelFrom the label of the channel 
#' to be displayed on the y axis:
#' the conventional syntax is : `channelName` + " - " + `channelMarker`
#' @param yChannelLabelTo should be equal to yChannelLabelFrom
#' (otherwise no plot is returned but NULL) 
#' @param interactive if TRUE, uses ggplot_shiny
#' @param useAllCells if TRUE, no subsampling will be done
#' @param nDisplayCells if useAllCells == FALSE, the number of subsampled cells
#' @param useFixedLinearRange if TRUE, all channels using a linear scale will
#' use a fixed range set by linearRange
#' @param linearRange set for all channels using a linear scale,
#' if useFixedLinearRange == TRUE
#' @param transfoListName if not set to " ", the transformation list 
#' (as an object name ending with "_obj", as referenced in the pipeline 
#' workflow) to be used for for display. 
#' @return a ggplot (or plotly if interactive = TRUE) object
#' @export
#'
#' @examples
#' 
#' # run CytoPipeline object first
#' 
#' outputDir <- base::tempdir()
#' 
#' 
#' rawDataDir <-
#'     system.file("extdata", package = "CytoPipeline")
#' experimentName <- "OMIP021_PeacoQC"
#' sampleFiles <- 
#'     file.path(
#'         rawDataDir, 
#'         list.files(rawDataDir, pattern = "Donor"))
#' jsonDir <- system.file("extdata", package = "CytoPipeline")
#' jsonPath <- file.path(jsonDir, "pipelineParams.json")
#' 
#' pipL2 <- CytoPipeline(
#'     jsonPath,
#'     experimentName = experimentName,
#'     sampleFiles = sampleFiles)
#' 
#' suppressWarnings(execute(
#'     pipL2,
#'     rmCache = TRUE,
#'     path = outputDir))
#'
#'
#' plotDiffFlowFrame(
#'     experimentNameFrom = experimentName,
#'     whichQueueFrom = "pre-processing",
#'     sampleFileFrom = 1,
#'     flowFrameNameFrom = "remove_doublets_obj",
#'     xChannelLabelFrom = "FSC-A : NA",
#'     yChannelLabelFrom = "SSC-A : NA",
#'     path = outputDir,
#'     experimentNameTo = experimentName,
#'     whichQueueTo = "pre-processing",
#'     sampleFileTo = 1,
#'     flowFrameNameTo = "remove_debris_obj",
#'     xChannelLabelTo = "FSC-A : NA",
#'     yChannelLabelTo = "SSC-A : NA",
#'     useAllCells = TRUE,
#'     nDisplayCells = 0,
#'     useFixedLinearRange = TRUE,
#'     linearRange = c(-100, 262144))
#' 
#' plotDiffFlowFrame(
#'     experimentNameFrom = experimentName,
#'     whichQueueFrom = "pre-processing",
#'     sampleFileFrom = 1,
#'     flowFrameNameFrom = "remove_doublets_obj",
#'     xChannelLabelFrom = "FSC-A : NA",
#'     yChannelLabelFrom = "SSC-A : NA",
#'     path = outputDir,
#'     experimentNameTo = experimentName,
#'     whichQueueTo = "pre-processing",
#'     sampleFileTo = 1,
#'     flowFrameNameTo = "remove_debris_obj",
#'     xChannelLabelTo = "FSC-A : NA",
#'     yChannelLabelTo = "SSC-A : NA",
#'     useAllCells = FALSE,
#'     nDisplayCells = 100,
#'     useFixedLinearRange = FALSE,
#'     linearRange = NULL)
#' 
#' plotDiffFlowFrame(
#'     experimentNameFrom = experimentName,
#'     whichQueueFrom = "pre-processing",
#'     sampleFileFrom = 1,
#'     flowFrameNameFrom = "remove_debris_obj",
#'     xChannelLabelFrom = "FSC-A : NA",
#'     yChannelLabelFrom = "Comp-525/50Violet-A : L/D Aqua - Viability",
#'     path = outputDir,
#'     experimentNameTo = experimentName,
#'     whichQueueTo = "pre-processing",
#'     sampleFileTo = 1,
#'     flowFrameNameTo = "remove_dead_cells_obj",
#'     xChannelLabelTo = "FSC-A : NA",
#'     yChannelLabelTo = "Comp-525/50Violet-A : L/D Aqua - Viability",
#'     useAllCells = TRUE,
#'     nDisplayCells = 0,
#'     useFixedLinearRange = FALSE,
#'     linearRange = NULL,
#'     transfoListName = "scale_transform_estimate_obj")   
#' 
plotDiffFlowFrame <- function(
        experimentNameFrom,
        experimentNameTo,
        whichQueueFrom,
        whichQueueTo,
        sampleFileFrom,
        sampleFileTo,
        path,
        flowFrameNameFrom,
        flowFrameNameTo,
        xChannelLabelFrom,
        xChannelLabelTo,
        yChannelLabelFrom,
        yChannelLabelTo,
        interactive = FALSE,
        useAllCells,
        nDisplayCells,
        useFixedLinearRange,
        linearRange,
        transfoListName = " ") {

    if (xChannelLabelFrom != " " &&
        yChannelLabelFrom != " " &&
        xChannelLabelTo == xChannelLabelFrom &&
        yChannelLabelTo == yChannelLabelFrom &&
        whichQueueFrom == whichQueueTo &&
        sampleFileFrom == sampleFileTo) {
        message("displaying flow frame comparison plot...")
        pipLFrom <- buildCytoPipelineFromCache(
            experimentNameFrom,
            path = path)

        if (is.character(sampleFileFrom)){
            if (!sampleFileFrom %in% sampleFiles(pipLFrom)){
                displayNames <- sampleDisplayNames(pipLFrom)
                if (sampleFileFrom %in% displayNames) {
                    sampleFileFrom <- 
                        sampleNameFromDisplayName(
                            pipLFrom,
                            sampleFileFrom)
                } # else do nothing
            }
        }

        ffFrom <- getCytoPipelineFlowFrame(
            pipLFrom,
            whichQueue = whichQueueFrom,
            sampleFile = sampleFileFrom,
            objectName = flowFrameNameFrom,
            path = path)

        pipLTo <- buildCytoPipelineFromCache(
            experimentNameTo,
            path = path)

        if (is.character(sampleFileTo)){
            if (!sampleFileTo %in% sampleFiles(pipLTo)){
                displayNames <- sampleDisplayNames(pipLTo)
                if (sampleFileTo %in% displayNames) {
                    sampleFileTo <- 
                        sampleNameFromDisplayName(
                            pipLTo,
                            sampleFileTo)
                } # else do nothing
            }
        }

        ffTo <- getCytoPipelineFlowFrame(
            pipLTo,
            whichQueue = whichQueueTo,
            sampleFile = sampleFileTo,
            objectName = flowFrameNameTo,
            path = path)

        transList <- NULL
        if (transfoListName != " ") {
            transList <-
                getCytoPipelineScaleTransform(
                    pipLFrom,
                    whichQueue = "scale transform",
                    objectName = transfoListName,
                    path = path)
        }

        xChannel <- channelLabel2Name(xChannelLabelFrom, ffFrom)
        if (length(xChannel) == 0) return()

        fluoChannels <- flowCore::colnames(ffFrom)[areFluoCols(ffFrom)]
        xScale <- "linear"

        if (xChannel %in% fluoChannels) {
            xScale <- "logicle"
        }

        yScale <- "linear"
        if (yChannelLabelFrom == " ") {
            yChannel <- NULL
        } else {
            yChannel <- channelLabel2Name(yChannelLabelFrom, ffFrom)
            if (length(yChannel) == 0) return()
            if (yChannel %in% fluoChannels) {
                yScale <- "logicle"
            }
        }

        if (useFixedLinearRange) {
            xLinearRange <- linearRange
            yLinearRange <- linearRange
        } else {
            xLinearRange <- NULL
            yLinearRange <- NULL
        }


        if (useAllCells) {
            nDisplayCells <- Inf
        }

        # used to prevent shiny app being frozen by plotly plot
        nEffectiveDisplayCells <- min(10000, nDisplayCells)

        p <- ggplotFilterEvents(
            ffPre = ffFrom,
            ffPost = ffTo,
            xChannel = xChannel,
            yChannel = yChannel,
            nDisplayCells = nEffectiveDisplayCells,
            seed = 0,
            size = 0.1,
            xScale = xScale,
            yScale = yScale,
            xLinearRange = xLinearRange,
            yLinearRange = yLinearRange,
            transList = transList,
            interactive = interactive)

        nEventFrom <- flowCore::nrow(ffFrom)
        nEventTo <- flowCore::nrow(ffTo)

        removedEvents <- 100 *
            (nEventFrom - nEventTo) / max(nEventFrom, nEventTo)
        if (experimentNameFrom == experimentNameTo &&
            sampleFileFrom == sampleFileTo &&
            whichQueueFrom == whichQueueTo &&
            whichQueueFrom == "pre-processing" &&
            nEventFrom >= nEventTo) {
            theSubtitle <- paste0(
                "removed ", round(removedEvents,2),
                "% of events, ", nEventTo,
                " good events remaining")
        } else {
            theSubtitle <- paste0(
                "nb events left: ", nEventFrom,
                ", nb events right: ", nEventTo,
                ", diff: ", round(abs(removedEvents), 2), "%")
        }

        if (interactive) {

            if (is.null(p)) {
                p <- plotly::plotly_empty(type="scatter", mode = "markers")
            } else {
                p <- plotly::layout(
                    p = plotly::ggplotly(p),
                    title = list(
                        text = theSubtitle,
                        xanchor = "center",
                        font = list(
                            size = 10)))
            }

        } else {
            p <- p + ggplot2::labs(subtitle = theSubtitle)
        }


        return(p)

    }
    return(NULL)
}

#' @title Plot a flow frame in 1D with explicit user given scale transform  
#' @description This function plots a 1D view, i.e. the marginal distribution
#' for one specified channel, of the given flow frame, using the specific 
#' user-provided scale transformation parameters.
#' @param ff the `flowFrame` to be plotted
#' @param channel the name of the channel of which to display the marginal 
#' distribution (i.e. the channel name used as column in the ff expression 
#' matrix).
#' @param applyTransform if "data", data are explicitly transformed using
#' the user provided sclae transformation parameters, before display
#' if "axis scale only" (default), the data are not transformed, 
#' i.e. only the x axis scale is defined according to 
#' the scale transformation parameters. 
#' @param transfoType the transformation type, currently only 
#' `linear` and `logicle`(bi-exponential) are supported.
#' @param linA the intercept parameter of the linear transformation.  
#' @param linB the slope parameter of the linear transformation. 
#' @param negDecades the number of additional decades on the negative side 
#' for the logicle transformation.  
#' @param width the width parameter of the logicle transformation. 
#' @param posDecades the number of positive decades of the logicle 
#' tranformation. 
#' @return a ggplot object
#' @export
#'
#' @examples
#' # run CytoPipeline object first
#' 
#' outputDir <- base::tempdir()
#' 
#' 
#' rawDataDir <-
#'     system.file("extdata", package = "CytoPipeline")
#' experimentName <- "OMIP021_PeacoQC"
#' sampleFiles <- 
#'     file.path(
#'         rawDataDir, 
#'         list.files(rawDataDir, pattern = "Donor"))
#' jsonDir <- system.file("extdata", package = "CytoPipeline")
#' jsonPath <- file.path(jsonDir, "pipelineParams.json")
#' 
#' pipL2 <- CytoPipeline(
#'     jsonPath,
#'     experimentName = experimentName,
#'     sampleFiles = sampleFiles)
#' 
#' suppressWarnings(execute(
#'     pipL2,
#'     rmCache = TRUE,
#'     path = outputDir))
#'
#' ff <- CytoPipeline::getCytoPipelineFlowFrame(
#' pipL2,
#' path = outputDir,
#' whichQueue = "scale transform",
#' objectName = "flowframe_aggregate_obj"
#' )
#' 
#' plotScaleTransformedChannel(
#'     ff,
#'     channel = "FSC-A",
#'     transfoType = "linear",
#'     linA = 0.0002,
#'     linB = -0.5)
#' 
#' plotScaleTransformedChannel(
#'     ff,
#'     channel = "Comp-670/30Violet-A",
#'     transfoType = "logicle",
#'     negDecades = 1,
#'     width = 0.5,
#'     posDecades = 4
#' )
#' 
#' plotScaleTransformedChannel(
#'     ff,
#'     channel = "CD3",
#'     applyTransform = "data",
#'     transfoType = "logicle",
#'     negDecades = 1,
#'     width = 0.5,
#'     posDecades = 4
#' )
#'
plotScaleTransformedChannel <- function(
        ff,
        channel,
        applyTransform =
            c("axis scale only", "data"),
        transfoType =
            c("linear", "logicle"),
        linA, linB,
        negDecades, width, posDecades) {
    applyTransform <- match.arg(applyTransform)
    transfoType <- match.arg(transfoType)
    if (!is.null(ff)) {
        if (transfoType == "linear") {
            theTrans <- flowCore::linearTransform(
                a = linA,
                b = linB)
        } else {
            theTrans <- flowCore::logicleTransform(
                w = width,
                m = posDecades + width,
                a = negDecades)
        }

        # find the channel name (can be based on marker name)
        ch <- flowCore::getChannelMarker(ff, channel)$name

        theTransList <- flowCore::transformList(
            from = ch,
            tfun = theTrans)

        if (applyTransform == "data") {
            runTransforms <- TRUE
            if (transfoType == "logicle") {
                linearRange <- c(0, posDecades + width)
            } else {
                linearRange <- NULL
            }

        } else {
            runTransforms <- FALSE
            linearRange <- NULL
        }

        ggplotEvents(
            ff,
            xChannel = ch,
            xLinearRange = linearRange,
            nDisplayCells = 10000,
            seed = 0,
            transList = theTransList,
            runTransforms = runTransforms,
            fill = "lightgreen",
            alpha = 0.2)
    }

}

# CytoPipelineGUI - Copyright (C) <2022-2025> 
# <Université catholique de Louvain (UCLouvain), Belgique>
#   
#   Description and complete License: see LICENSE file.
# 
# This program (CytoPipelineGUI) is free software: 
#   you can redistribute it and/or modify it under the terms of the GNU 
#   General Public License as published by the Free Software Foundation, 
#   either version 3 of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).


# run CytoPipeline object first

outputDir <- base::tempdir()

if (!interactive()) pdf(NULL)

rawDataDir <-
    system.file("extdata", package = "CytoPipeline")
experimentName <- "OMIP021_PeacoQC"
sampleFiles <- file.path(rawDataDir, list.files(rawDataDir,
                                                pattern = "Donor"))
jsonDir <- system.file("extdata", package = "CytoPipeline")
jsonPath <- file.path(jsonDir, "pipelineParams.json")

pipL2 <- CytoPipeline(jsonPath,
                      experimentName = experimentName,
                      sampleFiles = sampleFiles)

suppressWarnings(execute(pipL2,
                         rmCache = TRUE,
                         path = outputDir))

test_that("plotSelectedWorkflow works", {
    # pre-processing, existing sample file
    expect_no_error(
        plotSelectedWorkflow(
            experimentName = experimentName,
            whichQueue = "pre-processing",
            sampleFile = sampleFiles[1],
            path = outputDir))
    
    # pre-processing, existing sample file, by index
    expect_no_error(
        plotSelectedWorkflow(
            experimentName = experimentName,
            whichQueue = "pre-processing",
            sampleFile = 1,
            path = outputDir))
    
    # pre-processing, defaulted sample file
    expect_no_error(
        plotSelectedWorkflow(
            experimentName = experimentName,
            whichQueue = "pre-processing",
            sampleFile = NULL,
            path = outputDir))
        
    # scale transform => no comment for sample file not used
    expect_no_error(
        plotSelectedWorkflow(
            experimentName = experimentName,
            whichQueue = "scale transform",
            sampleFile = "any",
            path = outputDir))
    
    # pre-processing, sample file not found => no error message
    expect_no_error(
        plotSelectedWorkflow(
            experimentName = experimentName,
            whichQueue = "pre-processing",
            sampleFile = "any",
            path = outputDir))
        
})

test_that("plotSelectedFlowFrame works", {
    
    # TO DO : reactivate vdiffr tests when CytoPipeline dev version has been 
    # updated with the new OMIP021 dataset (version 1.2 in release or dev)
    # and remove expect_no_error()
    
    p1 <- plotSelectedFlowFrame(
        experimentName = experimentName,
        whichQueue = "pre-processing",
        sampleFile = 1,
        flowFrameName = "remove_debris_obj",
        path = outputDir,
        xChannelLabel = "FSC-A : NA",
        yChannelLabel = "SSC-A : NA",
        useAllCells = TRUE,
        nDisplayCells = 0,
        useFixedLinearRange = TRUE,
        linearRange = c(-100, 262144))
    
    vdiffr::expect_doppelganger(
        "pltSelFF-allc-fixlin-notran", fig = p1)  
    
    p2 <- plotSelectedFlowFrame(
        experimentName = experimentName,
        whichQueue = "pre-processing",
        sampleFile = 1,
        flowFrameName = "remove_debris_obj",
        path = outputDir,
        xChannelLabel = "FSC-A : NA",
        yChannelLabel = "SSC-A : NA",
        useAllCells = FALSE,
        nDisplayCells = 100,
        useFixedLinearRange = FALSE,
        linearRange = NULL)
    
    # vdiffr::expect_doppelganger(
    #     "pltSelFF-100c-nofixlin-notran", fig = p2)
    
    p3 <- plotSelectedFlowFrame(
        experimentName = experimentName,
        whichQueue = "pre-processing",
        sampleFile = 1,
        flowFrameName = "remove_debris_obj",
        path = outputDir,
        xChannelLabel = "Comp-670/30Violet-A : BV785 - CD3",
        yChannelLabel = "Comp-780/60Red-A : APCCy7 - CD4",
        useAllCells = TRUE,
        nDisplayCells = 0,
        useFixedLinearRange = FALSE,
        linearRange = NULL,
        transfoListName = "scale_transform_estimate_obj")
    
    vdiffr::expect_doppelganger(
        "pltSelFF-allc-tran", fig = p3)
    
})

test_that("plotSelectedDiffFlowFrame works", {
    
    # TO DO : reactivate vdiffr tests when CytoPipeline dev version has been 
    # updated with the new OMIP021 dataset (version 1.2 in release or dev)
    # and remove expect_no_error()
    
    expect_no_error({
        p1 <- CytoPipelineGUI:::plotDiffFlowFrame(
            experimentNameFrom = experimentName,
            whichQueueFrom = "pre-processing",
            sampleFileFrom = 1,
            flowFrameNameFrom = "remove_doublets_obj",
            xChannelLabelFrom = "FSC-A : NA",
            yChannelLabelFrom = "SSC-A : NA",
            path = outputDir,
            experimentNameTo = experimentName,
            whichQueueTo = "pre-processing",
            sampleFileTo = 1,
            flowFrameNameTo = "remove_debris_obj",
            xChannelLabelTo = "FSC-A : NA",
            yChannelLabelTo = "SSC-A : NA",
            useAllCells = TRUE,
            nDisplayCells = 0,
            useFixedLinearRange = TRUE,
            linearRange = c(-100, 262144))
        
        # vdiffr::expect_doppelganger(
        #     "pltDiffFF-allc-fixlin-notran", fig = p1)  
        
        p2 <- CytoPipelineGUI:::plotDiffFlowFrame(
            experimentNameFrom = experimentName,
            whichQueueFrom = "pre-processing",
            sampleFileFrom = 1,
            flowFrameNameFrom = "remove_doublets_obj",
            xChannelLabelFrom = "FSC-A : NA",
            yChannelLabelFrom = "SSC-A : NA",
            path = outputDir,
            experimentNameTo = experimentName,
            whichQueueTo = "pre-processing",
            sampleFileTo = 1,
            flowFrameNameTo = "remove_debris_obj",
            xChannelLabelTo = "FSC-A : NA",
            yChannelLabelTo = "SSC-A : NA",
            useAllCells = FALSE,
            nDisplayCells = 100,
            useFixedLinearRange = FALSE,
            linearRange = NULL)
        
        # vdiffr::expect_doppelganger(
        #     "pltDiffFF-100c-nofixlin-notran", fig = p2)
        
        p3 <- CytoPipelineGUI:::plotDiffFlowFrame(
            experimentNameFrom = experimentName,
            whichQueueFrom = "pre-processing",
            sampleFileFrom = 1,
            flowFrameNameFrom = "remove_debris_obj",
            xChannelLabelFrom = "FSC-A : NA",
            yChannelLabelFrom = "Comp-525/50Violet-A : L/D Aqua - Viability",
            path = outputDir,
            experimentNameTo = experimentName,
            whichQueueTo = "pre-processing",
            sampleFileTo = 1,
            flowFrameNameTo = "remove_dead_cells_obj",
            xChannelLabelTo = "FSC-A : NA",
            yChannelLabelTo = "Comp-525/50Violet-A : L/D Aqua - Viability",
            useAllCells = TRUE,
            nDisplayCells = 0,
            useFixedLinearRange = FALSE,
            linearRange = NULL,
            transfoListName = "scale_transform_estimate_obj")
        
        # suppressWarnings(vdiffr::expect_doppelganger(
        #     "pltDiffFF-allc-tran", fig = p3))
    })
    

    
})

test_that("plotScaleTransformedChannel works", {
    
    ff <- CytoPipeline::getCytoPipelineFlowFrame(
        pipL2,
        path = outputDir,
        whichQueue = "scale transform",
        objectName = "flowframe_aggregate_obj"
    )
    
    p1 <- plotScaleTransformedChannel(
        ff,
        channel = "FSC-A",
        transfoType = "linear",
        linA = 0.0002,
        linB = -0.5)
    
    # vdiffr::expect_doppelganger(
    #     "pltScaleTransCh-linear", fig = p1)
    
    p2 <- plotScaleTransformedChannel(
        ff,
        channel = "Comp-670/30Violet-A",
        transfoType = "logicle",
        negDecades = 1,
        width = 0.5,
        posDecades = 4
    )
    
    # vdiffr::expect_doppelganger(
    #     "pltScaleTransCh-logicle", fig = p2)
    
    p3 <- plotScaleTransformedChannel(
        ff,
        channel = "CD3",
        applyTransform = "data",
        transfoType = "logicle",
        negDecades = 1,
        width = 0.5,
        posDecades = 4
    )
    
    # vdiffr::expect_doppelganger(
    #     "pltScaleTransCh-logicle-data-mker", fig = p3)  
    
    
    
    
})

# CytoPipelineGUI - Copyright (C) <2022-2025> 
# <Université catholique de Louvain (UCLouvain), Belgique>
#   
#   Description and complete License: see LICENSE file.
# 
# This program (CytoPipelineGUI) is free software: 
#   you can redistribute it and/or modify it under the terms of the GNU 
#   General Public License as published by the Free Software Foundation, 
#   either version 3 of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).

#' @title interactive display and modification of scale transform list
#' @description this application allows the user to visualize a scale 
#' transformation list, possibly amending it channel after channel, 
#' and save the results on disk. 
#' The needed input tranformation list and flow frame for visualization needs
#' to be read from a CytoPipeline experiments stored in cache.
#' @param dir the root directory into which the engine will look for existing
#' CytoPipeline experiments
#' @import shiny
#' @import CytoPipeline
#' @return no return value
#' @export
#' @examples
#' 
#' # run CytoPipeline object first
#' 
#' outputDir <- base::tempdir()
#' 
#' 
#' rawDataDir <-
#'     system.file("extdata", package = "CytoPipeline")
#' experimentName <- "OMIP021_PeacoQC"
#' sampleFiles <- 
#'     file.path(rawDataDir, list.files(rawDataDir, pattern = "Donor"))
#' jsonDir <- system.file("extdata", package = "CytoPipeline")
#' jsonPath <- file.path(jsonDir, "pipelineParams.json")
#' 
#' pipL2 <- 
#'     CytoPipeline(
#'         jsonPath,
#'         experimentName = experimentName,
#'         sampleFiles = sampleFiles)
#' 
#' suppressWarnings(execute(
#'     pipL2,
#'     rmCache = TRUE,
#'     path = outputDir))
#'
#' # run shiny app
#'
#' if (interactive())
#'     ScaleTransformApp(dir = outputDir)                     
#' 
ScaleTransformApp <- function(dir = ".") {
    if (interactive()) {
        path <- dir
        
        # initialize input lists
        experimentNames <-
            getCytoPipelineExperimentNames(path = path)
        if (length(experimentNames) == 0) {
            stop("no experiment found in current directory!")
        }
        ui <- fluidPage(
            # App title ----
            titlePanel("Manual scale transformations adjustments"),
            scaleTransformUI(
                id = "scaleTransformUI",
                experimentNames = experimentNames,
                enableTransfoListUpdate = TRUE, 
                enableTransfoListSave = TRUE)
        )
        
        server <- function(input, output, session) {
            
            scaleTransformServer(
                id = "scaleTransformUI",
                path = path)
        }
        
        shinyApp(ui, server)
    }
    
}

#' @title interactive visualization of flow cytometry data analysis pipeline 
#' objects stored in cache
#'
#' @param dir the root directory into which the engine will look for existing
#' CytoPipeline experiments
#' @param debug if TRUE, will output messages on the console tracking the
#' shiny events, for debugging purposes
#' @return no return value
#' @import shiny
#' @import CytoPipeline
#' @export
#' @examples
#' 
#' # run CytoPipeline object first
#' 
#' outputDir <- base::tempdir()
#' 
#' 
#' rawDataDir <-
#'     system.file("extdata", package = "CytoPipeline")
#' experimentName <- "OMIP021_PeacoQC"
#' sampleFiles <- 
#'     file.path(
#'         rawDataDir, 
#'         list.files(
#'             rawDataDir, 
#'             pattern = "Donor"))
#' jsonDir <- system.file("extdata", package = "CytoPipeline")
#' jsonPath <- file.path(jsonDir, "pipelineParams.json")
#' 
#' pipL2 <- CytoPipeline(
#'     jsonPath,
#'     experimentName = experimentName,
#'     sampleFiles = sampleFiles)
#' 
#' suppressWarnings(execute(
#'     pipL2,
#'     rmCache = TRUE,
#'     path = outputDir))
#'
#' # run shiny app
#'
#' if (interactive())
#'     CytoPipelineCheckApp(dir = outputDir)                    
#'
CytoPipelineCheckApp <-  function(dir = ".", debug = FALSE) {
    if (interactive()) {
        path <- dir
        # initialize input lists
        experimentNames <-
            getCytoPipelineExperimentNames(path = path)
        if (length(experimentNames) == 0) {
            stop("no experiment found in current directory!")
        }
        debugFlag <- debug
        debugMessage <- function(msg){
            if (debugFlag) {
                message(msg)
            }
        }
        
        ui <- 
            navbarPage(
                "CytoPipeline check", id = "tabs",
                tabPanel(
                    "Experiments",
                    splitLayout(
                        fluidPage(
                            selectInput(
                                inputId = "experimentFrom",
                                label = "experiment:",
                                choices = experimentNames),
                            selectInput(
                                inputId = "whichQueueFrom",
                                label = "processing queue:",
                                choices = c("pre-processing",
                                            "scale transform")),
                            conditionalPanel(
                                condition = 
                                    "input.whichQueueFrom == 'pre-processing'",
                                selectInput(
                                    inputId = "sampleFrom",
                                    label = "sample:",
                                    choices = c(" "))
                            ),
                            plotOutput("workflowPlotFrom")
                        ),
                        fluidPage(
                            selectInput(
                                inputId = "experimentTo",
                                label = 
                                    "experiment (for comparisons):",
                                choices = experimentNames),
                            selectInput(
                                inputId = "whichQueueTo",
                                label = "processing queue:",
                                choices = c("pre-processing",
                                            "scale transform")),
                            conditionalPanel(
                                condition = 
                                    "input.whichQueueTo == 'pre-processing'",
                                selectInput(
                                    inputId = "sampleTo",
                                    label = "sample:",
                                    choices = c(" "))
                            ),
                            plotOutput("workflowPlotTo")
                            
                        )
                    )
                ),
                tabPanel(
                    "flow frames compare",
                    splitLayout(
                        fluidPage(
                            selectInput(
                                inputId = "flowFrameFrom",
                                label = "flow frame:",
                                choices = c(" ")),
                            selectInput(
                                inputId = "xchannelFrom",
                                label = "x channel/marker:",
                                choices = c(" ")),
                            selectInput(
                                inputId = "ychannelFrom",
                                label = "y channel/marker:",
                                choices = c(" ")),
                            plotOutput("ffPlotFrom")
                        ),
                        fluidPage(
                            selectInput(
                                inputId = "flowFrameTo",
                                label = "flow frame: (for comparison)",
                                choices = c(" ")),
                            selectInput(
                                inputId = "xchannelTo",
                                label = "x channel/marker:",
                                choices = c(" ")),
                            selectInput(
                                inputId = "ychannelTo",
                                label = "y channel/marker:",
                                choices = c(" ")),
                            plotOutput("ffPlotTo")
                        )
                    ),
                    
                    div(
                        plotly::plotlyOutput("ffDiffPlotly", width = "50%"), 
                        align = "center")
                ),
                tabPanel(
                    "scale transforms",
                    scaleTransformUI(
                        id = "scaleTransformUI",
                        experimentNames = experimentNames,
                        enableTransfoListUpdate = FALSE, 
                        enableTransfoListSave = FALSE),
                ),
                navbarMenu(
                    "More",
                    tabPanel(
                        "flowframe plots settings", 
                        fluidPage(
                            checkboxInput(
                                inputId = "useAllSampleEvents",
                                label = "Use all sample events:",
                                value = FALSE),
                            conditionalPanel(
                                condition = "!input.useAllSampleEvents",
                                numericInput(
                                    inputId = "nSubSampleEvents",
                                    label = 
                                        "Max nb of displayed events:",
                                    value = 10000,
                                    min = 100,
                                    max = NA)),
                            checkboxInput(
                                inputId = "useFixedLinearRange",
                                label = "Use fixed linear range:",
                                value = FALSE),
                            conditionalPanel(
                                condition = 
                                    "input.useFixedLinearRange",
                                numericInput(
                                    inputId = "minValueLinearRange",
                                    label = "Min:",
                                    value = -100,
                                    min = -50000,
                                    max = 262143),
                                numericInput(
                                    inputId = "maxValueLinearRange",
                                    label = "Max:",
                                    value = 262144,
                                    min = 100,
                                    max = NA)
                            ),
                            checkboxInput(
                                inputId = "useSelectedTransfo",
                                label = 
                                    "Use scale transformation list object:",
                                value = FALSE)
                        )),
                    #   "------",
                    #   tabPanel("About", "About page")
                )
                
            ) # end navbarPage
        
        
        server <- function(input, output, session) {
            
            observeEvent(input$experimentFrom, {
                debugMessage("obs event: experimentFrom")
                
                # update available processing queues, as well as
                # list of samples, based on selected experiment
                updateExperiment(
                    newExperimentName = input$experimentFrom,
                    whichQueueName = "whichQueueFrom",
                    currentWhichQueueValue = input$whichQueueFrom,
                    sampleInputId = "sampleFrom",
                    currentSampleValue = input$sampleFrom,
                    FFInputId = "flowFrameFrom",
                    currentFFValue = input$flowFrameFrom,
                    path = path)
                
                # update experiment for comparison 
                # (by default, but only if no currently selected experiment)
                if(input$experimentTo == " ") {
                    updateSelectInput(
                        inputId = "experimentTo",
                        selected = input$experimentFrom)
                }
                
                debugMessage("end obs event: experimentFrom")
            })
            
            observeEvent(input$experimentTo, {
                debugMessage("obs event: experimentTo")

                # update available processing queues, as well as
                # list of samples, based on selected experiment
                updateExperiment(
                    newExperimentName = input$experimentTo,
                    whichQueueName = "whichQueueTo",
                    currentWhichQueueValue = input$whichQueueTo,
                    sampleInputId = "sampleTo",
                    currentSampleValue = input$sampleTo,
                    FFInputId = "flowFrameTo",
                    currentFFValue = input$flowFrameTo,
                    path = path)
                
                debugMessage("end obs event: experimentTo")
            })
            
            observeEvent(input$whichQueueFrom, {
                debugMessage("obs event: whichQueueFrom")
                
                # update list of available FF objects
                
                updateFFList(
                    experimentName = input$experimentFrom,
                    whichQueue = input$whichQueueFrom,
                    sampleFile = input$sampleFrom,
                    path = path,
                    inputId = "flowFrameFrom",
                    currentValue = input$flowFrameFrom)
                
                # update which queue for comparison (default)
                updateSelectInput(
                    inputId = "whichQueueTo",
                    selected = input$whichQueueFrom)
                debugMessage("end obs event: whichQueueFrom")
            })
            
            observeEvent(input$whichQueueTo, {
                debugMessage("obs event: whichQueueTo")
                
                # update list of available FF objects
                
                updateFFList(
                    experimentName = input$experimentTo,
                    whichQueue = input$whichQueueTo,
                    sampleFile = input$sampleTo,
                    path = path,
                    inputId = "flowFrameTo",
                    currentValue = input$flowFrameTo)
                debugMessage("end obs event: whichQueueTo")
            })
            
            observeEvent(input$sampleFrom, {
                debugMessage("obs event: sampleFrom")
                # update list of available FF objects
                updateFFList(
                    experimentName = input$experimentFrom,
                    whichQueue = input$whichQueueFrom,
                    sampleFile = input$sampleFrom,
                    path = path,
                    inputId = "flowFrameFrom",
                    currentValue = input$flowFrameFrom)
                
                # update sample for comparison (default)
                updateSelectInput(
                    inputId = "sampleTo",
                    selected = input$sampleFrom)
                debugMessage("end obs event: sampleFrom")
            })
            
            observeEvent(input$sampleTo, {
                debugMessage("obs event: sampleTo")
                # update list of available FF objects
                updateFFList(
                    experimentName = input$experimentTo,
                    whichQueue = input$whichQueueTo,
                    sampleFile = input$sampleTo,
                    path = path,
                    inputId = "flowFrameTo",
                    currentValue = input$flowFrameTo)
                debugMessage("end obs event: sampleTo")
            })
            
            observeEvent(input$flowFrameFrom, {
                debugMessage("obs event: flowFrameFrom")
                # update list of channels
                updateChannelMarkerList(
                    experimentName = input$experimentFrom,
                    whichQueue = input$whichQueueFrom,
                    sampleFile = input$sampleFrom,
                    path = path,
                    flowFrameName = input$flowFrameFrom,
                    inputIds = c(
                        "xchannelFrom",
                        "ychannelFrom"),
                    currentValues = c(
                        input$xchannelFrom,
                        input$ychannelFrom))
                debugMessage("end obs event: flowFrameFrom")
            })
            
            observeEvent(input$flowFrameTo, {
                debugMessage("obs event: flowFrameTo")
                # update list of channels
                updateChannelMarkerList(
                    experimentName = input$experimentTo,
                    whichQueue = input$whichQueueTo,
                    sampleFile = input$sampleTo,
                    path = path,
                    flowFrameName = input$flowFrameTo,
                    inputIds = c(
                        "xchannelTo",
                        "ychannelTo"),
                    currentValues = c(
                        input$xchannelTo,
                        input$ychannelTo))
                debugMessage("end obs event: flowFrameTo")
            })
            
            observeEvent(input$xchannelFrom, {
                debugMessage("obs event: xchannelFrom")
                # update xchannel for comparison (default)
                updateSelectInput(
                    inputId = "xchannelTo",
                    selected = input$xchannelFrom)
                debugMessage("end obs event: xchannelFrom")
            })
            
            observeEvent(input$ychannelFrom, {
                debugMessage("obs event: ychannelFrom")
                # update ychannel for comparison (default)
                updateSelectInput(
                    inputId = "ychannelTo",
                    selected = input$ychannelFrom)
                debugMessage("end obs event: ychannelFrom")
            })
            
            observeEvent(input$useAllSampleEvents, {
                debugMessage(paste0("obs event: useAllSampleEvents"))
            })
            
            output$workflowPlotFrom <- renderPlot({
                debugMessage("rendering workflow plot (from)")
                try(
                    plotSelectedWorkflow(
                        experimentName = input$experimentFrom,
                        whichQueue = input$whichQueueFrom,
                        sampleFile = input$sampleFrom,
                        path = path)
                )
            })
            
            output$workflowPlotTo <- renderPlot({
                debugMessage("rendering workflow plot (to)")
                try(
                    plotSelectedWorkflow(
                        experimentName = input$experimentTo,
                        whichQueue = input$whichQueueTo,
                        sampleFile = input$sampleTo,
                        path = path)
                )
            })
            
            
            output$ffPlotFrom <- renderPlot({
                transfoListName <- " "
                if (input$useSelectedTransfo){
                    transfoListName <- 
                        input[[NS("scaleTransformUI", "scaleTransfoList")]]
                }
                plotSelectedFlowFrame(
                    experimentName = input$experimentFrom,
                    whichQueue = input$whichQueueFrom,
                    sampleFile = input$sampleFrom,
                    path = path,
                    flowFrameName = input$flowFrameFrom,
                    xChannelLabel = input$xchannelFrom,
                    yChannelLabel = input$ychannelFrom,
                    useAllCells = input$useAllSampleEvents,
                    nDisplayCells = input$nSubSampleEvents,
                    useFixedLinearRange =
                        input$useFixedLinearRange,
                    linearRange = c(
                        input$minValueLinearRange,
                        input$maxValueLinearRange),
                    transfoListName = transfoListName)
            })
            
            output$ffPlotTo <- renderPlot({
                transfoListName <- " "
                if (input$useSelectedTransfo){
                    transfoListName <- 
                        input[[NS("scaleTransformUI", "scaleTransfoList")]]
                }
                plotSelectedFlowFrame(
                    experimentName = input$experimentTo,
                    whichQueue = input$whichQueueTo,
                    sampleFile = input$sampleTo,
                    path = path,
                    flowFrameName = input$flowFrameTo,
                    xChannelLabel = input$xchannelTo,
                    yChannelLabel = input$ychannelTo,
                    useAllCells = input$useAllSampleEvents,
                    nDisplayCells = input$nSubSampleEvents,
                    useFixedLinearRange =
                        input$useFixedLinearRange,
                    linearRange = c(input$minValueLinearRange,
                                    input$maxValueLinearRange),
                    transfoListName = transfoListName)
            })
            
            output$ffDiffPlotly <- plotly::renderPlotly({
                transfoListName <- " "
                if (input$useSelectedTransfo){
                    transfoListName <- 
                        input[[NS("scaleTransformUI", "scaleTransfoList")]]
                }
                plotDiffFlowFrame(
                    experimentNameFrom = input$experimentFrom,
                    experimentNameTo = input$experimentTo,
                    whichQueueFrom = input$whichQueueFrom,
                    whichQueueTo = input$whichQueueTo,
                    sampleFileFrom = input$sampleFrom,
                    sampleFileTo = input$sampleTo,
                    path = path,
                    flowFrameNameFrom = input$flowFrameFrom,
                    flowFrameNameTo = input$flowFrameTo,
                    xChannelLabelFrom = input$xchannelFrom,
                    xChannelLabelTo = input$xchannelTo,
                    yChannelLabelFrom = input$ychannelFrom,
                    yChannelLabelTo = input$ychannelTo,
                    interactive = TRUE,
                    useAllCells = input$useAllSampleEvents,
                    nDisplayCells = input$nSubSampleEvents,
                    useFixedLinearRange =
                        input$useFixedLinearRange,
                    linearRange = c(input$minValueLinearRange,
                                    input$maxValueLinearRange),
                    transfoListName = transfoListName)
            })
            
            scaleTransformServer(
                id = "scaleTransformUI",
                path = path)
            
            
        } # end main server
        shinyApp(ui, server)
    }
    
}

## -----------------------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("CytoPipelineGUI")


## ----pkg, include = FALSE-----------------------------------------------------
library(CytoPipeline)
library(CytoPipelineGUI)
library(patchwork)

## ----preparation, message=FALSE, warning=FALSE--------------------------------
# raw data
rawDataDir <- system.file("extdata", package = "CytoPipeline")
sampleFiles <- file.path(rawDataDir, list.files(rawDataDir,
                                                pattern = "Donor"))
# output files
workDir <- suppressMessages(base::tempdir())
# pipeline configuration files (in json)
jsonDir <- rawDataDir

# creation of CytoPipeline objects

pipL_PeacoQC <-
  CytoPipeline(file.path(jsonDir, "OMIP021_PeacoQC_pipeline.json"),
               experimentName = "OMIP021_PeacoQC",
               sampleFiles = sampleFiles)

pipL_flowAI <-
  CytoPipeline(file.path(jsonDir, "OMIP021_flowAI_pipeline.json"),
               experimentName = "OMIP021_flowAI",
               sampleFiles = sampleFiles)

# execute PeacoQC pipeline
suppressWarnings(execute(pipL_PeacoQC, rmCache = TRUE, path = workDir))

# execute flowAI pipeline
suppressWarnings(execute(pipL_flowAI, rmCache = TRUE, path = workDir))

## ----scaleTransformQueueDisplay, results='markup', fig.cap="Scale transform processing queue", echo=FALSE, out.width='75%', fig.align='center', fig.wide = TRUE----
knitr::include_graphics("figs/scaleTransformQueue.png", error = FALSE)

## ----preProcessingQueueDisplay, results='markup', fig.cap="Pre-processing queue for two different pipeline settings", echo=FALSE, out.width='100%', fig.align='center', fig.wide = TRUE----
knitr::include_graphics("figs/preProcessingQueues.png", error = FALSE)

## ----flowframe_viz------------------------------------------------------------
if (interactive()) {
    CytoPipelineGUI::CytoPipelineCheckApp(dir = workDir)    
}

## ----workflow_viz_prog, out.height=450, out.width=600, fig.height=4.5, fig.width=6, fig.align='center'----
# pre-processing workflow
expName <- "OMIP021_PeacoQC"
CytoPipelineGUI::plotSelectedWorkflow(
            experimentName = expName,
            whichQueue = "pre-processing",
            sampleFile = sampleFiles[1],
            path = workDir)

## ----flowframe_viz_prog, out.height=300, out.width=900, fig.height=6, fig.width=18, fig.align='center', message = FALSE----
expName1 <- "OMIP021_PeacoQC"
expName2 <- "OMIP021_flowAI"

p1 <- CytoPipelineGUI::plotSelectedFlowFrame(
    experimentName = expName1,
    whichQueue = "pre-processing",
    sampleFile = 2,
    flowFrameName = "perform_QC_obj",
    path = workDir,
    xChannelLabel = "Time : NA",
    yChannelLabel = "FSC-A : NA",
    useAllCells = TRUE,
    useFixedLinearRange = FALSE)

p2 <- CytoPipelineGUI::plotSelectedFlowFrame(
    experimentName = expName2,
    whichQueue = "pre-processing",
    sampleFile = 2, 
    flowFrameName = "perform_QC_obj",
    path = workDir,
    xChannelLabel = "Time : NA",
    yChannelLabel = "FSC-A : NA",
    useAllCells = TRUE,
    useFixedLinearRange = FALSE)

p3 <- CytoPipelineGUI::plotDiffFlowFrame(
    path = workDir,
    experimentNameFrom = expName1,
    whichQueueFrom = "pre-processing",
    sampleFileFrom = 2, 
    flowFrameNameFrom = "perform_QC_obj",
    xChannelLabelFrom = "Time : NA",
    yChannelLabelFrom = "FSC-A : NA",
    experimentNameTo = expName2,
    whichQueueTo = "pre-processing",
    sampleFileTo = 2,
    flowFrameNameTo = "perform_QC_obj",
    xChannelLabelTo = "Time : NA",
    yChannelLabelTo = "FSC-A : NA",
    useAllCells = TRUE,
    useFixedLinearRange = FALSE)

p1+p2+p3


## ----scale_transform_viz------------------------------------------------------
# 5. show scale transformations
if (interactive()){
    CytoPipelineGUI::ScaleTransformApp(dir = workDir)    
}

## ----scale_transform_viz_prog, out.height=300, out.width=600, fig.height=6, fig.width=12, fig.align='center', message = FALSE----
expName <- "OMIP021_PeacoQC"
pipL <- CytoPipeline::buildCytoPipelineFromCache(
    experimentName = expName,
    path = workDir
)

    ff <- CytoPipeline::getCytoPipelineFlowFrame(
        pipL,
        path = workDir,
        whichQueue = "scale transform",
        objectName = "flowframe_aggregate_obj"
    )
    
    p1 <- plotScaleTransformedChannel(
        ff,
        channel = "FSC-A",
        transfoType = "linear",
        linA = 0.0002,
        linB = -0.5)
    
    p2 <- plotScaleTransformedChannel(
        ff,
        channel = "CD3",
        applyTransform = "data",
        transfoType = "logicle",
        negDecades = 1,
        width = 0.5,
        posDecades = 4
    )
    
    p1+p2

## ----sessioninfo, echo=FALSE--------------------------------------------------
sessionInfo()

# CytoPipelineGUI - Copyright (C) <2022-2025>
# <Université catholique de Louvain (UCLouvain), Belgique>
#
#   Description and complete License: see LICENSE file.
#
# This program (CytoPipelineGUI) is free software:
#   you can redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).

#' @title CytoPipelineGUI package
#' 
#' @name CytoPipelineGUI
#'
#' @rdname CytoPipelineGUI
#' 
#' @seealso \link[CytoPipeline:CytoPipeline-package]{CytoPipeline}
#' @seealso [CytoPipelineGUI::CytoPipelineCheckApp] 
#' @seealso [CytoPipelineGUI::ScaleTransformApp]
#'
#' @description
#'
#' `CytoPipelineGUI` is the companion package of `CytoPipeline`, 
#' and is used for interactive visualization. 
#' It implements two shiny applications :
#' - a shiny app for interactive comparison of flow frames 
#'   that are the results of CytoProcessingSteps of the same 
#'   or different CytoPipeline experiments.  
#' It is launched using the following statement: `CytoPipelineCheckApp()`  
#' - a shiny app for interactive visualization and manual adjustments of scale 
#'   transformation objects. It is launched using the following statement: 
#'   `ScaleTransformApp()` 
#'   
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

# CytoPipelineGUI - Copyright (C) <2022-2025> 
# <Université catholique de Louvain (UCLouvain), Belgique>
#   
#   Description and complete License: see LICENSE file.
# 
# This program (CytoPipelineGUI) is free software: 
#   you can redistribute it and/or modify it under the terms of the GNU 
#   General Public License as published by the Free Software Foundation, 
#   either version 3 of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).

scaleTransformUI <- function(
        id,
        experimentNames, 
        enableTransfoListUpdate = FALSE,
        enableTransfoListSave = FALSE) {
    tagList(
        # Sidebar layout with input and output definitions ----
        sidebarLayout(
            
            # Sidebar panel for inputs ----
            sidebarPanel(
                selectInput(
                    inputId = NS(id, "experimentTransfo"),
                    label = "experiment:",
                    choices = experimentNames),
                
                # Input: the scale transformation list we would like to
                # display
                selectInput(
                    inputId = NS(id, "scaleTransfoList"),
                    label = "scale transform list",
                    choices = c(" ")),
                
                # Input: the flow frame object, from which we display
                # the distributions
                selectInput(
                    inputId = NS(id, "flowFrameTransfo"),
                    label = "flow frame:",
                    choices = c(" ")),
                
                # Input: the active channel
                selectInput(
                    inputId = NS(id, "transfoChannel"),
                    label = "channel/marker:",
                    choices = c(" ")),
                
                # Input : transform data or scale only ?
                selectInput(
                    inputId = NS(id, "applyTransform"),
                    label = "Visualize transformation on:",
                    choices = c("axis scale only", "data")),
                
                # Input: type of transformation : linear or logicle
                selectInput(
                    inputId = NS(id, "transfoType"),
                    label = "Transformation type:",
                    choices = c("linear", "logicle")),
                
                # Input: set of inputs for linear transformations
                conditionalPanel(
                    condition = "input.transfoType == 'linear'",
                    ns = NS(id),
                    # condition = 
                    #     paste0("input.", 
                    #            NS(id, "transfoType"), " == 'linear'"),
                    numericInput(
                        inputId = NS(id, "linA"),
                        label = "slope",
                        value = 1., min = 0., 
                        max = 1000000, step = NA),
                    numericInput(
                        inputId = NS(id, "linB"),
                        label = "intercept",
                        value = 0., min = -Inf, 
                        max = Inf, step = NA)),
                
                # Input: set of inputs for conditional transformations
                conditionalPanel(
                    condition = "input.transfoType == 'logicle'",
                    ns = NS(id),
                    # condition = 
                    #     paste0("input.",
                    #            NS(id, "transfoType"), " == 'logicle'"),
                    sliderInput(
                        inputId = NS(id, "negDecades"),
                        label = "Extra Neg Decades:",
                        min = 0,
                        max = 1,
                        step = 0.1,
                        value = 0),
                    sliderInput(
                        inputId = NS(id, "width"),
                        label = "Width:",
                        min = 0,
                        max = 3,
                        step = 0.1,
                        value = 1),
                    sliderInput(
                        inputId = NS(id, "posDecades"),
                        label = "Pos Decades:",
                        min = 2,
                        max = 7,
                        step = 0.1,
                        value = 4.5)
                ),
                
                conditionalPanel(
                    condition = tolower(as.character(enableTransfoListUpdate)),
                    actionButton(
                        inputId = NS(id, "apply"),
                        label = "Apply to channel")),
                
                conditionalPanel(
                    condition = tolower(as.character(enableTransfoListSave)),
                    downloadButton(
                        outputId = NS(id, "save"),
                        label = "Save")),
                
                # conditionalPanel(
                #   condition = tolower(
                #       as.character(transListSource == "input")),
                #   actionButton(inputId = NS(id, "done"),
                #                label = "Done!")),
                
            ), # end sidebar panel
            
            # Main panel for displaying outputs ----
            mainPanel(
                
                # Output: Histogram ----
                plotOutput(outputId = NS(id, "distPlot"))
                
            )
        ) # end sidebarLayout
    ) # end taglist
}

scaleTransformServer <- function(id, path, transfoList = NULL, ff = NULL) {
    moduleServer(id, function(input, output, session) {
        ##### SCALE TRANSFORM PART ###################################
        
        # stores currently obtained scale transfo list
        currentTransfoList <- NULL
        
        if (!is.null(transfoList)) {
            if (is.null(ff)) {
                stop(
                    "if transfoList parameter is passed as non null, ",
                    "so must be ff parameter")
            } 
        } else {
            if (!is.null(ff)) {
                stop(
                    "if ff parameter is passed as non null, ",
                    "so must be transfoList parameter")
            }
        }
        
        # note the following statement also holds when the RHS is NULL
        currentTransfoList <- transfoList
        
        currentFFFull <- reactive({
            ### Warning: activating messages in reactive disturbs the process!!
            if (!is.null(ff)) {
                ff
            } else {
                if (input$flowFrameTransfo == " ") {
                    NULL
                } else if (grepl(
                    pattern = "Synth. agg. - ", 
                    x = input$flowFrameTransfo)) {
                    # no flow frame in the scale transform queue
                    # => build synthetic one from pre-processing queue
                    # => sample 10,000 events from max. 5 flow frames
                    nTotalEvents <- 10000
                    nFilesMax <- 5
                    
                    objectName <- sub(
                        pattern = "Synth. agg. - ", 
                        replacement = "",
                        x = input$flowFrameTransfo)
                    
                    pipL <- buildCytoPipelineFromCache(
                        input$experimentTransfo,
                        path = path)
                    samples <- sampleFiles(pipL)
                    nSamples <- length(samples)
                    if (nSamples >= nFilesMax) {
                        samples <- 
                            samples[sample.int(nSamples, size = nFilesMax)]
                    }
                    ffList <- list()
                    for (s in samples) {
                        df <- getCytoPipelineObjectInfos(
                            pipL,
                            whichQueue = "pre-processing",
                            sampleFile = s,
                            path = path)
                        if (nrow(df[(
                            df$ObjectClass == "flowFrame" | 
                            df$ObjectClass == "cytoframe") & 
                            df$ObjectName == objectName,]) > 0) {
                            myff <- getCytoPipelineFlowFrame(
                                pipL,
                                whichQueue = "pre-processing",
                                sampleFile = s,
                                objectName = objectName,
                                path = path)
                            
                            ffList <- c(ffList, myff)
                        }
                    }
                    if (length(ffList) == 0)
                        NULL
                    
                    fs <- flowCore::flowSet(ffList)
                    ff <- 
                        CytoPipeline::aggregateAndSample(
                            fs,
                            nTotalEvents = nTotalEvents,
                            seed = 0)
                    ff
                } else {
                    pipL <- 
                        buildCytoPipelineFromCache(
                            input$experimentTransfo,
                            path = path)
                    
                    df <- getCytoPipelineObjectInfos(
                        pipL,
                        whichQueue = "scale transform",
                        sampleFile = NULL,
                        path = path)
                    if (nrow(df[(
                        df$ObjectClass == "flowFrame" | 
                        df$ObjectClass == "cytoframe") & 
                        df$ObjectName == input$flowFrameTransfo,
                    ]) > 0) {
                        ff <- 
                            getCytoPipelineFlowFrame(
                                pipL,
                                whichQueue = "scale transform",
                                sampleFile = NULL,
                                objectName = input$flowFrameTransfo,
                                path = path)
                        
                        ff
                    } else {
                        NULL
                    }
                } 
            }
        })
        
        currentFF <- reactive({
            if (is.null(currentFFFull)) {
                NULL
            } else if (input$transfoChannel != " ") {
                channel <- 
                    channelLabel2Name(input$transfoChannel, currentFFFull())
                currentFFFull()[, channel]
            } else {
                NULL
            }
        })
        
        r <- reactiveValues(
            transfoType = "linear",
            linA = 1.0,
            linB = 0.0,
            negDecades = 0.0,
            width = 1,
            posDecades = 4.5)
        
        getTransfoList <- function(
            experimentName,
            path,
            transfoName) {
            
            if (transfoName != " ") {
                pipL <- buildCytoPipelineFromCache(
                    experimentName,
                    path = path)
                
                transfoList <-
                    getCytoPipelineScaleTransform(
                        pipL,
                        whichQueue = "scale transform",
                        objectName = transfoName,
                        path = path)
                return(transfoList)
            } else {
                return(NULL)
            }
        }
        
        updateTransfoParams <- function(
            transfoList,
            ff) {
            if (!is.null(transfoList) && !is.null(ff)) {
                tPars <- getTransfoParams(
                    transfoList,
                    channel = flowCore::colnames(ff)[1])
                if (is.null(tPars)) {
                    # trial with "Comp-"
                    tPars <- 
                        getTransfoParams(
                            transfoList,
                            channel = paste0(
                                "Comp-",
                                flowCore::colnames(ff)[1]))
                }
                
                if (!is.null(tPars)) {
                    updateSelectInput(
                        inputId = "transfoType",
                        selected = tPars$type)
                    
                    if (tPars$type == "logicle") {
                        val_negDecades <- round(tPars$paramsList$a, digits = 1)
                        val_width <- round(tPars$paramsList$w, digits = 1)
                        val_posDecades <- 
                            round(
                                tPars$paramsList$m -
                                    tPars$paramsList$w, digits = 1)
                        
                        updateSliderInput(
                            inputId = "negDecades",
                            value = val_negDecades)
                        updateSliderInput(
                            inputId = "width",
                            value = val_width)
                        updateSliderInput(
                            inputId = "posDecades",
                            value = val_posDecades)
                        
                        synchronizeReactiveTransfoParams(
                            transfoType = tPars$type,
                            negDecades = val_negDecades,
                            width = val_width,
                            posDecades = val_posDecades)
                    } else {
                        val_a <- signif(tPars$paramsList$a, 3)
                        val_b <- signif(tPars$paramsList$b, 3)
                        
                        updateNumericInput(
                            inputId = "linA",
                            value = val_a)
                        updateNumericInput(
                            inputId = "linB",
                            value = val_b)
                        
                        synchronizeReactiveTransfoParams(
                            transfoType = tPars$type,
                            linA = val_a,
                            linB = val_b)
                    }
                    
                }
                
            }
        }
            
        
        synchronizeReactiveTransfoParams <- function(
            transfoType,
            linA = 0.,
            linB = 0.,
            negDecades = 0.,
            width = 0.,
            posDecades = 0.){
            if (transfoType != r$transfoType) {
                #message("transfoType is different => updating...")
                r$transfoType <- transfoType
            }
            if (transfoType == "linear") {
                if (linA != r$linA) {
                    #message("linA is different => updating...")
                    r$linA <- linA
                }
                if (linB != r$linB) {
                    #message("linB is different => updating...")
                    r$linB <- linB
                }
            } else {
                if (negDecades != r$negDecades) {
                    #message("negDecades is different => updating...")
                    r$negDecades <- negDecades
                }
                if (width != r$width) {
                    #message("width is different => updating...")
                    r$width <- width
                }
                if (posDecades != r$posDecades) {
                    #message("posDecades is different => updating...")
                    r$posDecades <- posDecades
                }
            }
        }
        
        observeEvent(input$experimentTransfo, {
            #message("obs event: experimentTransfo")
            # update list of available transList objects 
            # for scale transfo display
            updateTransList(
                experimentName = input$experimentTransfo,
                path = path,
                inputId = "scaleTransfoList",
                currentValue = input$scaleTransfoList)
            
            # update list of available FF objects for scale transfo display
            updateFFListForTransDisplay(
                experimentName = input$experimentTransfo,
                path = path,
                inputId = "flowFrameTransfo",
                currentValue = input$flowFrameTransfo)
            
            #message("end obs event: experimentTransfo")
        })
        
        observeEvent(input$flowFrameTransfo, {
            #message("obs event: flowFrameTransfo")
            # update list of channels
            ffName <- input$flowFrameTransfo
            if (grepl(
                    pattern = "Synth. agg. - ", 
                    x = ffName)) {
                ffName <- sub(
                    pattern = "Synth. agg. - ", 
                    replacement = "",
                    x = ffName)  
                updateChannelMarkerList(
                    experimentName = input$experimentTransfo,
                    whichQueue = "pre-processing",
                    sampleFile = 1,
                    path = path,
                    flowFrameName = ffName,
                    inputIds = c("transfoChannel"),
                    currentValues = c(input$transfoChannel))
                
            } else {
                updateChannelMarkerList(
                    experimentName = input$experimentTransfo,
                    whichQueue = "scale transform",
                    sampleFile = NULL,
                    path = path,
                    flowFrameName = ffName,
                    inputIds = c("transfoChannel"),
                    currentValues = c(input$transfoChannel))
            }
            
            #message("end obs event: flowFrameTransfo")
        })
        
        observeEvent(input$scaleTransfoList, {
            #message("obs event: scaleTransfoList")
            
            assign(
                "currentTransfoList", 
                getTransfoList(
                    input$experimentTransfo,
                    path = path,
                    transfoName = input$scaleTransfoList),
                inherits = TRUE)
            
            updateTransfoParams(
                currentTransfoList,
                currentFF())  
            #message("end obs event: scaleTransfoList")
        })
        
        observeEvent(input$transfoChannel, {
            #message("obs event: transfoChannel")
            updateTransfoParams(
                currentTransfoList,
                currentFF())
        })
        
        
        observeEvent({
            input$transfoType
            input$linA
            input$linB
            input$negDecades
            input$width
            input$posDecades
        },{
            #message(paste0("obs event: one transfo param"))
            synchronizeReactiveTransfoParams(
                transfoType = req(input$transfoType),
                linA = req(input$linA),
                linB = req(input$linB),
                negDecades = req(input$negDecades),
                width = req(input$width),
                posDecades = req(input$posDecades))
            #message(paste0("end obs event: one transfo param"))
        })
        
        observeEvent(input$apply, {
            #message("obs event: apply")
            
            if (!is.null(currentTransfoList) && !is.null(currentFF())) {
                
                myList <- isolate(reactiveValuesToList(r))
                
                if (r$transfoType == "linear") {
                    newTrans <- 
                        flowCore::linearTransform(
                            a = myList$linA,
                            b = myList$linB)
                } else {
                    newTrans <- 
                        flowCore::logicleTransform(
                            w = myList$width,
                            m = myList$posDecades + 
                                myList$width,
                            a = myList$negDecades)
                }
                
                channel <- flowCore::colnames(currentFF())[1]
                
                localTransfoList <- currentTransfoList
                localTransfoList@transforms[[channel]] <- NULL
                localTransfoList <-
                    c(  localTransfoList, 
                        flowCore::transformList(channel, newTrans))
                
                assign(
                    "currentTransfoList",
                    localTransfoList,
                    inherits = TRUE)
            }
            
        })
        
        output$distPlot <- renderPlot({
            message("Rendering scale transformed channel...")
            channel <- flowCore::colnames(currentFF())[1]
            plotScaleTransformedChannel(
                ff = currentFF(),
                channel = channel,
                applyTransform = input$applyTransform,
                transfoType = r$transfoType,
                linA = r$linA,
                linB = r$linB,
                negDecades = r$negDecades,
                width = r$width,
                posDecades = r$posDecades)
            
        })
        
        output$save <- downloadHandler(
            filename = function() {
                paste0(path, "/", "savedScaleTranfoList", ".rds")
            },
            content = function(file) {
                saveRDS(currentTransfoList, file = file)
            }
        )
    })
}

# CytoPipelineGUI - Copyright (C) <2022-2025> 
# <Université catholique de Louvain (UCLouvain), Belgique>
#   
#   Description and complete License: see LICENSE file.
# 
# This program (CytoPipelineGUI) is free software: 
#   you can redistribute it and/or modify it under the terms of the GNU 
#   General Public License as published by the Free Software Foundation, 
#   either version 3 of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).

channelLabel2Name <- function(channelLabel, ff) {
    channels <- unname(flowCore::colnames(ff))
    chLabels <- lapply(
        channels,
        FUN = function(ch){
            chMk <- flowCore::getChannelMarker(ff, ch)
            lab <- paste0(chMk$name, " : ", chMk$desc)
            lab
        })
    
    channel <- channels[which(chLabels == channelLabel)]
    return(channel)
}

updateExperiment <- function(
        newExperimentName,
        whichQueueName,
        currentWhichQueueValue,
        sampleInputId,
        currentSampleValue,
        FFInputId,
        currentFFValue,
        path) {
    #browser()    
    pipL <- buildCytoPipelineFromCache(
        newExperimentName,
        path = path)
    
    whichQueueChoices <- character()
    
    nPreProcessingSteps <- 
        getNbProcessingSteps(
            pipL, 
            whichQueue = "pre-processing")
    if (nPreProcessingSteps > 0) {
        whichQueueChoices <- 
            c(whichQueueChoices, "pre-processing")
    }
    
    nScaleTransfoSteps <- 
        getNbProcessingSteps(
            pipL,
            whichQueue = "scale transform")
    if (nScaleTransfoSteps > 0) {
        whichQueueChoices <- 
            c(whichQueueChoices, "scale transform")
    }
    
    if (length(whichQueueChoices) == 0)
        stop(
            "experiment without any processing step selected ",
            "=> inconsistency")
    
    newWQSelection <- currentWhichQueueValue
    if (!(newWQSelection %in% whichQueueChoices)) {
        newWQSelection <- whichQueueChoices[1]
    }
    
    updateSelectInput(
        inputId = whichQueueName,
        choices = whichQueueChoices,
        selected = newWQSelection)
    
    sampleDisplays <- sampleDisplayNames(pipL)
    
    if (length(sampleDisplays) == 0) {
        # specific case when no sample has been found in the cache
        # e.g. scale transform only experiment
        newSampleChoices <- " "
    } else {
        newSampleChoices <- c(" ", sampleDisplays)
    }
    
    newSampleSelection <- currentSampleValue
    if (!(newSampleSelection %in% newSampleChoices)) {
        newSampleSelection <- " "
    }
    updateSelectInput(
        inputId = sampleInputId,
        choices = newSampleChoices,
        selected = newSampleSelection)
    
    # update list of available FF objects
    # note we force sampleFile the new selection
    # because it is automatically the current choice, 
    # and to avoid ghost sample file flowing
    updateFFList(
        experimentName = newExperimentName,
        whichQueue = newWQSelection,
        sampleFile = newSampleSelection,
        path = path,
        inputId = FFInputId,
        currentValue = currentFFValue)
}

updateFFList <- function(
        experimentName,
        whichQueue,
        sampleFile,
        path,
        inputId,
        currentValue){
    #message("Updating FF list...")
    noFF <- FALSE
    if (!is.null(sampleFile)){
        if (sampleFile == " ") {
            if (whichQueue == "pre-processing") {
                noFF <- TRUE
            } else {
                sampleFile <- NULL
            }
        }
    } 
    
    if (noFF) {
        newChoices <- " "
    } else {
        pipL <- buildCytoPipelineFromCache(
            experimentName,
            path = path)
        if (!is.null(sampleFile)){
            sampleFile <- sampleNameFromDisplayName(pipL, sampleFile)
        }
        df <- getCytoPipelineObjectInfos(
            pipL,
            whichQueue = whichQueue,
            sampleFile = sampleFile,
            path = path)
        FFNames <-
            df[
                df$ObjectClass == "flowFrame" | df$ObjectClass == "cytoframe",
                "ObjectName"]
        newChoices <- c(" ", FFNames)
    }
    
    newSelection <- currentValue
    if (!(newSelection %in% newChoices)) {
        newSelection <- " "
    }
    updateSelectInput(
        inputId = inputId, 
        choices = newChoices,
        selected = newSelection)
    
}

updateFFListForTransDisplay <- function(
        experimentName,
        path,
        inputId,
        currentValue){
    #message("Updating FF list for trans list display...")
    pipL <- buildCytoPipelineFromCache(
        experimentName,
        path = path)
    df <- getCytoPipelineObjectInfos(
        pipL,
        whichQueue = "scale transform",
        sampleFile = NULL,
        path = path)
    FFNames <-
        df[
            df$ObjectClass == "flowFrame" | df$ObjectClass == "cytoframe",
            "ObjectName"]
    
    if (length(FFNames) == 0) {
        # no flow frame in the scale transform queue
        # => build synthetic one from pre-processing queue
        df <- getCytoPipelineObjectInfos(
            pipL,
            whichQueue = "pre-processing",
            sampleFile = 1,
            path = path)
        FFNames <- 
            df[
                df$ObjectClass == "flowFrame" | df$ObjectClass == "cytoframe",
                "ObjectName"]
        FFNames <- paste0("Synth. agg. - ", FFNames)
    }
    newChoices <- c(" ", FFNames)
    
    
    newSelection <- currentValue
    if (!(newSelection %in% newChoices)) {
        newSelection <- " "
    }
    updateSelectInput(
        inputId = inputId, 
        choices = newChoices,
        selected = newSelection)
}


updateTransList <- function(
        experimentName,
        path,
        inputId,
        currentValue){
    if (experimentName != " ") {
        #message("Updating trans list...")
        pipL <- buildCytoPipelineFromCache(
            experimentName,
            path = path)
        df <- getCytoPipelineObjectInfos(
            pipL,
            whichQueue = "scale transform",
            sampleFile = NULL,
            path = path)
        scaleTransfoNames <-
            df[df$ObjectClass == "transformList", "ObjectName"]
        
        newChoices <- c(" ", scaleTransfoNames)
        
        newSelection <- currentValue
        if (!(newSelection %in% newChoices)) {
            newSelection <- " "
        }
        
        updateSelectInput(
            inputId = inputId, 
            choices = newChoices,
            selected = newSelection)
    }
}

updateChannelMarkerList <- function(
        experimentName,
        whichQueue,
        sampleFile,
        path,
        flowFrameName,
        inputIds,
        currentValues){
    if (flowFrameName != " ") {
        #message("Updating channel marker list")
        pipL <- buildCytoPipelineFromCache(
            experimentName,
            path = path)
        if (!is.null(sampleFile)) {
            sampleFile <- sampleNameFromDisplayName(pipL, sampleFile)
        }
        ff <- getCytoPipelineFlowFrame(
            pipL,
            whichQueue = whichQueue,
            sampleFile = sampleFile,
            objectName = flowFrameName,
            path = path)
        
        # sort out channels and and channel_labels
        channels <- unname(flowCore::colnames(ff)[areSignalCols(ff)])
        
        # add time channel
        timeCh <- findTimeChannel(ff)
        
        channels <- c(channels, timeCh)
        
        chLabels <- lapply(
            channels,
            FUN = function(ch){
                chMk <- flowCore::getChannelMarker(ff, ch)
                lab <- paste0(chMk$name, " : ", chMk$desc)
                lab
            })
        for (i in seq_along(inputIds)) {
            newChoices <- c(" ", chLabels)
            
            newSelection <- currentValues[i]
            if (!(newSelection %in% newChoices)) {
                newSelection <- " "
            }
            
            updateSelectInput(
                inputId = inputIds[i],
                choices = newChoices,
                selected = newSelection)
        }
        
    } else {
        for (inputId in inputIds) {
            updateSelectInput(
                inputId = inputId,
                choices = c(" "))
        }
    }
    
    
}

---
title: "CytoPipelineGUI : visualization of Flow Cytometry Data Analysis Pipelines run with CytoPipeline"
author:
    - name: Philippe Hauchamps
    - name: Laurent Gatto
package: CytoPipelineGUI
abstract: >
 This vignette gives a short introduction to CytoPipelineGUI, which is the companion package of CytoPipeline for interactive visualization of flow cytometry data pre-processing pipeline results.
 This vignette is distributed under a CC BY-SA license.
output:
  BiocStyle::html_document:
    toc_float: true
bibliography: CytoPipeline.bib    
vignette: >
  %\VignetteIndexEntry{CytoPipelineGUI : visualization of Flow Cytometry Data Analysis Pipelines run with CytoPipeline}
  %\VignetteEngine{knitr::rmarkdown}
  %%\VignetteKeywords{FlowCytometry, Preprocessing, QualityControl, WorkflowStep, Software, Visualization}
  %\VignetteEncoding{UTF-8}
---

# Installation

To install this package, start R and enter (uncommented):

```{r}
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("CytoPipelineGUI")

```

# Foreword - Preparation of pipeline results to be visualized

`CytoPipelineGUI` is the companion package of `CytoPipeline`, and is used for
interactive visualization of flow cytometry data pre-processing pipeline 
results. It implements two shiny applications :  

- a shiny app for interactive comparison of flow frames that are the results 
of CytoProcessingSteps of the same or different CytoPipeline experiments.  
It is launched using the following statement: `CytoPipelineCheckApp()` 
(see below);  

- a shiny app for interactive visualization and manual adjustments of scale 
transformation objects. It is launched using the following statement: 
`ScaleTransformApp()` (see below).

In order to be able to show `CytoPipelineGUI` in action, as a pre-requisite 
we need to have created a `CytoPipeline` object, 
defined the different pipeline steps, and run the pipeline until completion,
so that all intermediate results can be found on a cache. 
These preliminary steps are performed by the preparation code below. 

```{r pkg, include = FALSE}
library(CytoPipeline)
library(CytoPipelineGUI)
library(patchwork)
```

```{r preparation, message=FALSE, warning=FALSE}
# raw data
rawDataDir <- system.file("extdata", package = "CytoPipeline")
sampleFiles <- file.path(rawDataDir, list.files(rawDataDir,
                                                pattern = "Donor"))
# output files
workDir <- suppressMessages(base::tempdir())
# pipeline configuration files (in json)
jsonDir <- rawDataDir

# creation of CytoPipeline objects

pipL_PeacoQC <-
  CytoPipeline(file.path(jsonDir, "OMIP021_PeacoQC_pipeline.json"),
               experimentName = "OMIP021_PeacoQC",
               sampleFiles = sampleFiles)

pipL_flowAI <-
  CytoPipeline(file.path(jsonDir, "OMIP021_flowAI_pipeline.json"),
               experimentName = "OMIP021_flowAI",
               sampleFiles = sampleFiles)

# execute PeacoQC pipeline
suppressWarnings(execute(pipL_PeacoQC, rmCache = TRUE, path = workDir))

# execute flowAI pipeline
suppressWarnings(execute(pipL_flowAI, rmCache = TRUE, path = workDir))
```


If you are unfamiliar with `CytoPipeline` package, and you would like to 
know more about these steps, it is advised that you read 
the `CytoPipeline` vignette, and/or that you watch the videos illustrating 
the `CytoPipeline` suite, which are accessible 
through links included in the `Demo.Rmd` vignette.



# Introduction

The visualization tools shown here are demonstrated on the results 
of two different previously run `CytoPipeline` objects. 
These flow cytometry pre-processing pipeline are described in details
in the `CytoPipeline` vignette. Here below is a short summary 
of the illustrating dataset, as well as the pipeline steps.


## Example dataset (more details in CytoPipeline vignette)
The example dataset that will be used throughout this vignette is derived from 
a reference public dataset accompanying the OMIP-021 (Optimized Multicolor 
Immunofluorescence Panel 021) article [@Gherardin2014-pj].  

A sub-sample of this public dataset is built-in in the `CytoPipeline` 
package, as the OMIP021 dataset. 
See the `MakeOMIP021Samples.R` script for more details 
on how the `OMIP021` dataset was created. This script is to be found 
in the `script` subdirectory in the `CytoPipeline` package installation path.

## Example of pre-processing and QC pipelines (more details in CytoPipeline vignette)
In our example pipeline, we assumed that we wanted to pre-process 
the two samples of the `OMIP021` dataset, and that we wanted to compare 
what we would obtain when pre-processing these files 
using two different QC methods.    

In the first pre-processing pipeline, we used the `flowAI` QC method 
[@Monaco2016-vo], while in the second pipeline, we used the `PeacoQC` method 
[@Emmaneel2021-xy]. 

In both pipelines, the first part consisted in estimating appropriate scale
transformation functions for all channels present in the sample `flowFrame`.
For this, we ran the following steps (Fig. 1):   

- reading the two samples `.fcs` files
- removing the margin events from each file
- applying compensation for each file
- aggregating and sub-sampling from each file
- estimating the scale transformations from the aggregated 
and sub-sampled data   

```{r scaleTransformQueueDisplay, results='markup', fig.cap="Scale transform processing queue", echo=FALSE, out.width='75%', fig.align='center', fig.wide = TRUE}
knitr::include_graphics("figs/scaleTransformQueue.png", error = FALSE)
```

After this first part, pre-processing for each file, one by one, was performed.  
However, depending on the choice of QC method, the order of steps
needed to be slightly different (see Fig. 2) : 

```{r preProcessingQueueDisplay, results='markup', fig.cap="Pre-processing queue for two different pipeline settings", echo=FALSE, out.width='100%', fig.align='center', fig.wide = TRUE}
knitr::include_graphics("figs/preProcessingQueues.png", error = FALSE)
```


# Interactive visualizations

## Visualizing pipeline runs at different steps

Using the `CytoPipelineGUI` package, it is possible to interactively inspect
intermediate results produced during the pipeline execution.  

This is done through the `CytoPipelineCheckApp`, which can provide 
a view of the data structure, i.e. the `flowFrame`, 
at any step of any pipeline, as well as a comparison between any the pair of 
`flowFrame` state.

```{r flowframe_viz}
if (interactive()) {
    CytoPipelineGUI::CytoPipelineCheckApp(dir = workDir)    
}
```

It is difficult to extensively demonstrate specific user interactions 
 in a vignette, therefore live demo videos can be found from the `Demo.Rmd` 
 vignette.

However, it is possible to mimic the call to some of the shiny application 
features, by using some specific `CytoPipelineGUI` exported functions. 

A first example below is a function call which retrieves 
the visuals of the workflow of a previously run pipeline:

```{r workflow_viz_prog, out.height=450, out.width=600, fig.height=4.5, fig.width=6, fig.align='center'}
# pre-processing workflow
expName <- "OMIP021_PeacoQC"
CytoPipelineGUI::plotSelectedWorkflow(
            experimentName = expName,
            whichQueue = "pre-processing",
            sampleFile = sampleFiles[1],
            path = workDir)
```

It is also possible to programmatically obtain comparison plots that 
are displayed within the shiny application.  
Here below is an example, where one is comparing the two pipelines 
(*PeacoQC* vs *flowAI*) after the QC step:

```{r flowframe_viz_prog, out.height=300, out.width=900, fig.height=6, fig.width=18, fig.align='center', message = FALSE}
expName1 <- "OMIP021_PeacoQC"
expName2 <- "OMIP021_flowAI"

p1 <- CytoPipelineGUI::plotSelectedFlowFrame(
    experimentName = expName1,
    whichQueue = "pre-processing",
    sampleFile = 2,
    flowFrameName = "perform_QC_obj",
    path = workDir,
    xChannelLabel = "Time : NA",
    yChannelLabel = "FSC-A : NA",
    useAllCells = TRUE,
    useFixedLinearRange = FALSE)

p2 <- CytoPipelineGUI::plotSelectedFlowFrame(
    experimentName = expName2,
    whichQueue = "pre-processing",
    sampleFile = 2, 
    flowFrameName = "perform_QC_obj",
    path = workDir,
    xChannelLabel = "Time : NA",
    yChannelLabel = "FSC-A : NA",
    useAllCells = TRUE,
    useFixedLinearRange = FALSE)

p3 <- CytoPipelineGUI::plotDiffFlowFrame(
    path = workDir,
    experimentNameFrom = expName1,
    whichQueueFrom = "pre-processing",
    sampleFileFrom = 2, 
    flowFrameNameFrom = "perform_QC_obj",
    xChannelLabelFrom = "Time : NA",
    yChannelLabelFrom = "FSC-A : NA",
    experimentNameTo = expName2,
    whichQueueTo = "pre-processing",
    sampleFileTo = 2,
    flowFrameNameTo = "perform_QC_obj",
    xChannelLabelTo = "Time : NA",
    yChannelLabelTo = "FSC-A : NA",
    useAllCells = TRUE,
    useFixedLinearRange = FALSE)

p1+p2+p3

```

## Visualization of scale transformations

Besides the flowFrame comparison tool, `CytoPipelineGUI` provides another 
shiny app, which allows to interactively visualize and manage 
the scale transformations that are generated as part of our prep-processing 
pipelines.  

If the shape of the scale transformations that were automatically set by the
chosen algorithm appears to be non satisfactory, it is possible, using this
shiny application, to manually adjust the parameters of the transformation,
and save the results in a RDS object. This object can then be re-used in another
pipeline instance.

```{r scale_transform_viz}
# 5. show scale transformations
if (interactive()){
    CytoPipelineGUI::ScaleTransformApp(dir = workDir)    
}
```

Note that here also, it is possible to obtain the visuals 
of the scale transformations programmatically, 
although this is a bit more evolved, as one has to use `CytoPipeline` functions 
for this.

```{r scale_transform_viz_prog, out.height=300, out.width=600, fig.height=6, fig.width=12, fig.align='center', message = FALSE}
expName <- "OMIP021_PeacoQC"
pipL <- CytoPipeline::buildCytoPipelineFromCache(
    experimentName = expName,
    path = workDir
)

    ff <- CytoPipeline::getCytoPipelineFlowFrame(
        pipL,
        path = workDir,
        whichQueue = "scale transform",
        objectName = "flowframe_aggregate_obj"
    )
    
    p1 <- plotScaleTransformedChannel(
        ff,
        channel = "FSC-A",
        transfoType = "linear",
        linA = 0.0002,
        linB = -0.5)
    
    p2 <- plotScaleTransformedChannel(
        ff,
        channel = "CD3",
        applyTransform = "data",
        transfoType = "logicle",
        negDecades = 1,
        width = 0.5,
        posDecades = 4
    )
    
    p1+p2
```



# Session information {-}

```{r sessioninfo, echo=FALSE}
sessionInfo()
```

# References {-}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plotSelectedFlowFrame}
\alias{plotSelectedFlowFrame}
\title{Plot a flow frame from a CytoPipeline run}
\usage{
plotSelectedFlowFrame(
  experimentName,
  whichQueue,
  sampleFile,
  flowFrameName,
  path,
  xChannelLabel,
  yChannelLabel,
  useAllCells,
  nDisplayCells,
  useFixedLinearRange,
  linearRange,
  transfoListName = " "
)
}
\arguments{
\item{experimentName}{the experiment name (representing a pipeline run)
from which to extract the flow frame}

\item{whichQueue}{"pre-processing" or "scale transform"}

\item{sampleFile}{in case 'whichQueue' is set to 'pre-processing, which
sample file to look at. This can be a number or a character.
\itemize{
\item if whichQueue == "scale transform", the sampleFile is ignored
\item if NULL and whichQueue == "pre-processing", the sampleFile is defaulted
to the first one belonging to the experiment
}}

\item{flowFrameName}{the name of the object to fetch
(as referenced in the pipeline workflow)}

\item{path}{the root path to look for the CytoPipeline experiment cache}

\item{xChannelLabel}{the label of the channel to be displayed on the x axis:
the conventional syntax is : \code{channelName} + " - " + \code{channelMarker}}

\item{yChannelLabel}{the label of the channel to be displayed on the y axis:
the conventional syntax is : \code{channelName} + " - " + \code{channelMarker}}

\item{useAllCells}{if TRUE, no subsampling will be done}

\item{nDisplayCells}{if useAllCells == FALSE, the number of subsampled cells}

\item{useFixedLinearRange}{if TRUE, all channels using a linear scale will
use a fixed range set by linearRange}

\item{linearRange}{set for all channels using a linear scale,
if useFixedLinearRange == TRUE}

\item{transfoListName}{if not set to " ", the transformation list
(as an object name ending with "_obj", as referenced in the pipeline
workflow) to be used for for display.}
}
\value{
a ggplot object
}
\description{
Based on an experiment name, this function will gather the
required flowFrame from the CytoPipeline disk cache and display it using
the user chosen 1D or 2D view.
}
\examples{

# run CytoPipeline object first

outputDir <- base::tempdir()


rawDataDir <-
    system.file("extdata", package = "CytoPipeline")
experimentName <- "OMIP021_PeacoQC"
sampleFiles <- 
    file.path(
        rawDataDir, 
        list.files(rawDataDir, pattern = "Donor"))
jsonDir <- system.file("extdata", package = "CytoPipeline")
jsonPath <- file.path(jsonDir, "pipelineParams.json")

pipL2 <- CytoPipeline(
    jsonPath,
    experimentName = experimentName,
    sampleFiles = sampleFiles)

suppressWarnings(execute(
    pipL2,
    rmCache = TRUE,
    path = outputDir))

plotSelectedFlowFrame(
    experimentName = experimentName,
    whichQueue = "pre-processing",
    sampleFile = 1,
    flowFrameName = "remove_debris_obj",
    path = outputDir,
    xChannelLabel = "FSC-A : NA",
    yChannelLabel = "SSC-A : NA",
    useAllCells = TRUE,
    nDisplayCells = 0,
    useFixedLinearRange = TRUE,
    linearRange = c(-100, 262144))

plotSelectedFlowFrame(
    experimentName = experimentName,
    whichQueue = "pre-processing",
    sampleFile = 1,
    flowFrameName = "remove_debris_obj",
    path = outputDir,
    xChannelLabel = "FSC-A : NA",
    yChannelLabel = "SSC-A : NA",
    useAllCells = FALSE,
    nDisplayCells = 100,
    useFixedLinearRange = FALSE,
    linearRange = NULL)

plotSelectedFlowFrame(
    experimentName = experimentName,
    whichQueue = "pre-processing",
    sampleFile = 1,
    flowFrameName = "remove_debris_obj",
    path = outputDir,
    xChannelLabel = "Comp-670/30Violet-A : BV785 - CD3",
    yChannelLabel = "Comp-780/60Red-A : APCCy7 - CD4",
    useAllCells = TRUE,
    nDisplayCells = 0,
    useFixedLinearRange = FALSE,
    linearRange = NULL,
    transfoListName = "scale_transform_estimate_obj")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{CytoPipelineCheckApp}
\alias{CytoPipelineCheckApp}
\title{interactive visualization of flow cytometry data analysis pipeline
objects stored in cache}
\usage{
CytoPipelineCheckApp(dir = ".", debug = FALSE)
}
\arguments{
\item{dir}{the root directory into which the engine will look for existing
CytoPipeline experiments}

\item{debug}{if TRUE, will output messages on the console tracking the
shiny events, for debugging purposes}
}
\value{
no return value
}
\description{
interactive visualization of flow cytometry data analysis pipeline
objects stored in cache
}
\examples{

# run CytoPipeline object first

outputDir <- base::tempdir()


rawDataDir <-
    system.file("extdata", package = "CytoPipeline")
experimentName <- "OMIP021_PeacoQC"
sampleFiles <- 
    file.path(
        rawDataDir, 
        list.files(
            rawDataDir, 
            pattern = "Donor"))
jsonDir <- system.file("extdata", package = "CytoPipeline")
jsonPath <- file.path(jsonDir, "pipelineParams.json")

pipL2 <- CytoPipeline(
    jsonPath,
    experimentName = experimentName,
    sampleFiles = sampleFiles)

suppressWarnings(execute(
    pipL2,
    rmCache = TRUE,
    path = outputDir))

# run shiny app

if (interactive())
    CytoPipelineCheckApp(dir = outputDir)                    

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CytoPipelineGUI-package.R
\docType{package}
\name{CytoPipelineGUI}
\alias{CytoPipelineGUI-package}
\alias{CytoPipelineGUI}
\title{CytoPipelineGUI package}
\description{
\code{CytoPipelineGUI} is the companion package of \code{CytoPipeline},
and is used for interactive visualization.
It implements two shiny applications :
\itemize{
\item a shiny app for interactive comparison of flow frames
that are the results of CytoProcessingSteps of the same
or different CytoPipeline experiments.
It is launched using the following statement: \code{CytoPipelineCheckApp()}
\item a shiny app for interactive visualization and manual adjustments of scale
transformation objects. It is launched using the following statement:
\code{ScaleTransformApp()}
}
}
\seealso{
\link[CytoPipeline:CytoPipeline-package]{CytoPipeline}

\link{CytoPipelineCheckApp}

\link{ScaleTransformApp}
}
\author{
\strong{Maintainer}: Philippe Hauchamps \email{philippe.hauchamps@uclouvain.be} (\href{https://orcid.org/0000-0003-2865-1852}{ORCID})

Authors:
\itemize{
  \item Laurent Gatto \email{laurent.gatto@uclouvain.be} (\href{https://orcid.org/0000-0002-1520-2268}{ORCID})
}

Other contributors:
\itemize{
  \item Dan Lin \email{dan.8.lin@gsk.com} [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plotDiffFlowFrame}
\alias{plotDiffFlowFrame}
\title{Plot the difference plot between two flow frames
from a CytoPipeline run}
\usage{
plotDiffFlowFrame(
  experimentNameFrom,
  experimentNameTo,
  whichQueueFrom,
  whichQueueTo,
  sampleFileFrom,
  sampleFileTo,
  path,
  flowFrameNameFrom,
  flowFrameNameTo,
  xChannelLabelFrom,
  xChannelLabelTo,
  yChannelLabelFrom,
  yChannelLabelTo,
  interactive = FALSE,
  useAllCells,
  nDisplayCells,
  useFixedLinearRange,
  linearRange,
  transfoListName = " "
)
}
\arguments{
\item{experimentNameFrom}{the experiment name (representing a pipeline run)
from which to extract the flow frame ('from' situation)}

\item{experimentNameTo}{the experiment name (representing a pipeline run)
from which to extract the flow frame ('to' situation)}

\item{whichQueueFrom}{"pre-processing" or "scale transform"
('from' situation)}

\item{whichQueueTo}{"pre-processing" or "scale transform"
('to' situation)}

\item{sampleFileFrom}{in case 'whichQueueFrom' is set to 'pre-processing,
which sample file to look at for the 'from' situation.
This can be a number or a character.
\itemize{
\item if whichQueueFrom == "scale transform", the sampleFileFrom is ignored
\item if NULL and whihQueueFrom == "pre-processing", the sampleFileFrom
is defaulted to the first one belonging to the experiment
}}

\item{sampleFileTo}{same as sampleFileFrom, but for the 'to' situation}

\item{path}{the root path to look for the CytoPipeline experiment cache}

\item{flowFrameNameFrom}{for the 'from' situation,
the name of the object to fetch (as referenced in the pipeline workflow)}

\item{flowFrameNameTo}{for the 'to' situation,
the name of the object to fetch (as referenced in the pipeline workflow)}

\item{xChannelLabelFrom}{the label of the channel
to be displayed on the x axis:
the conventional syntax is : \code{channelName} + " - " + \code{channelMarker}}

\item{xChannelLabelTo}{should be equal to xChannelLabelFrom
(otherwise no plot is returned but NULL)}

\item{yChannelLabelFrom}{the label of the channel
to be displayed on the y axis:
the conventional syntax is : \code{channelName} + " - " + \code{channelMarker}}

\item{yChannelLabelTo}{should be equal to yChannelLabelFrom
(otherwise no plot is returned but NULL)}

\item{interactive}{if TRUE, uses ggplot_shiny}

\item{useAllCells}{if TRUE, no subsampling will be done}

\item{nDisplayCells}{if useAllCells == FALSE, the number of subsampled cells}

\item{useFixedLinearRange}{if TRUE, all channels using a linear scale will
use a fixed range set by linearRange}

\item{linearRange}{set for all channels using a linear scale,
if useFixedLinearRange == TRUE}

\item{transfoListName}{if not set to " ", the transformation list
(as an object name ending with "_obj", as referenced in the pipeline
workflow) to be used for for display.}
}
\value{
a ggplot (or plotly if interactive = TRUE) object
}
\description{
Based on an experiment name, this function will gather the
required flowFrames from the CytoPipeline disk cache and display
a difference plot using the user chosen 1D or 2D view.
}
\examples{

# run CytoPipeline object first

outputDir <- base::tempdir()


rawDataDir <-
    system.file("extdata", package = "CytoPipeline")
experimentName <- "OMIP021_PeacoQC"
sampleFiles <- 
    file.path(
        rawDataDir, 
        list.files(rawDataDir, pattern = "Donor"))
jsonDir <- system.file("extdata", package = "CytoPipeline")
jsonPath <- file.path(jsonDir, "pipelineParams.json")

pipL2 <- CytoPipeline(
    jsonPath,
    experimentName = experimentName,
    sampleFiles = sampleFiles)

suppressWarnings(execute(
    pipL2,
    rmCache = TRUE,
    path = outputDir))


plotDiffFlowFrame(
    experimentNameFrom = experimentName,
    whichQueueFrom = "pre-processing",
    sampleFileFrom = 1,
    flowFrameNameFrom = "remove_doublets_obj",
    xChannelLabelFrom = "FSC-A : NA",
    yChannelLabelFrom = "SSC-A : NA",
    path = outputDir,
    experimentNameTo = experimentName,
    whichQueueTo = "pre-processing",
    sampleFileTo = 1,
    flowFrameNameTo = "remove_debris_obj",
    xChannelLabelTo = "FSC-A : NA",
    yChannelLabelTo = "SSC-A : NA",
    useAllCells = TRUE,
    nDisplayCells = 0,
    useFixedLinearRange = TRUE,
    linearRange = c(-100, 262144))

plotDiffFlowFrame(
    experimentNameFrom = experimentName,
    whichQueueFrom = "pre-processing",
    sampleFileFrom = 1,
    flowFrameNameFrom = "remove_doublets_obj",
    xChannelLabelFrom = "FSC-A : NA",
    yChannelLabelFrom = "SSC-A : NA",
    path = outputDir,
    experimentNameTo = experimentName,
    whichQueueTo = "pre-processing",
    sampleFileTo = 1,
    flowFrameNameTo = "remove_debris_obj",
    xChannelLabelTo = "FSC-A : NA",
    yChannelLabelTo = "SSC-A : NA",
    useAllCells = FALSE,
    nDisplayCells = 100,
    useFixedLinearRange = FALSE,
    linearRange = NULL)

plotDiffFlowFrame(
    experimentNameFrom = experimentName,
    whichQueueFrom = "pre-processing",
    sampleFileFrom = 1,
    flowFrameNameFrom = "remove_debris_obj",
    xChannelLabelFrom = "FSC-A : NA",
    yChannelLabelFrom = "Comp-525/50Violet-A : L/D Aqua - Viability",
    path = outputDir,
    experimentNameTo = experimentName,
    whichQueueTo = "pre-processing",
    sampleFileTo = 1,
    flowFrameNameTo = "remove_dead_cells_obj",
    xChannelLabelTo = "FSC-A : NA",
    yChannelLabelTo = "Comp-525/50Violet-A : L/D Aqua - Viability",
    useAllCells = TRUE,
    nDisplayCells = 0,
    useFixedLinearRange = FALSE,
    linearRange = NULL,
    transfoListName = "scale_transform_estimate_obj")   

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plotSelectedWorkflow}
\alias{plotSelectedWorkflow}
\title{Plot a pipeline workflow from a CytoPipeline run}
\usage{
plotSelectedWorkflow(experimentName, whichQueue, sampleFile, path = path)
}
\arguments{
\item{experimentName}{the experiment name (representing a pipeline run)
from which to extract the workflow}

\item{whichQueue}{"pre-processing" or "scale transform"}

\item{sampleFile}{in case 'whichQueue' is set to 'pre-processing, which
sample file to look at. This can be a number or a character.
\itemize{
\item if whichQueue == "scale transform", the sampleFile is ignored
\item if NULL and whichQueue == "pre-processing", the sampleFile is defaulted
to the first one belonging to the experiment
}}

\item{path}{the root path to look for the CytoPipeline experiment cache}
}
\value{
nothing, but displays the plot as a side effect
}
\description{
Plot a pipeline workflow from a CytoPipeline run
}
\examples{

# run CytoPipeline object first

outputDir <- base::tempdir()


rawDataDir <-
    system.file("extdata", package = "CytoPipeline")
experimentName <- "OMIP021_PeacoQC"
sampleFiles <- 
    file.path(
        rawDataDir, 
        list.files(rawDataDir, pattern = "Donor"))
jsonDir <- system.file("extdata", package = "CytoPipeline")
jsonPath <- file.path(jsonDir, "pipelineParams.json")

pipL2 <- CytoPipeline(
    jsonPath,
    experimentName = experimentName,
    sampleFiles = sampleFiles)

suppressWarnings(execute(
    pipL2,
    rmCache = TRUE,
    path = outputDir))

plotSelectedWorkflow(
    experimentName = experimentName,
    whichQueue = "pre-processing",
    sampleFile = sampleFiles[1],
    path = outputDir)
    
plotSelectedWorkflow(
    experimentName = experimentName,
    whichQueue = "scale transform",
    sampleFile = NULL,
    path = outputDir)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{ScaleTransformApp}
\alias{ScaleTransformApp}
\title{interactive display and modification of scale transform list}
\usage{
ScaleTransformApp(dir = ".")
}
\arguments{
\item{dir}{the root directory into which the engine will look for existing
CytoPipeline experiments}
}
\value{
no return value
}
\description{
this application allows the user to visualize a scale
transformation list, possibly amending it channel after channel,
and save the results on disk.
The needed input tranformation list and flow frame for visualization needs
to be read from a CytoPipeline experiments stored in cache.
}
\examples{

# run CytoPipeline object first

outputDir <- base::tempdir()


rawDataDir <-
    system.file("extdata", package = "CytoPipeline")
experimentName <- "OMIP021_PeacoQC"
sampleFiles <- 
    file.path(rawDataDir, list.files(rawDataDir, pattern = "Donor"))
jsonDir <- system.file("extdata", package = "CytoPipeline")
jsonPath <- file.path(jsonDir, "pipelineParams.json")

pipL2 <- 
    CytoPipeline(
        jsonPath,
        experimentName = experimentName,
        sampleFiles = sampleFiles)

suppressWarnings(execute(
    pipL2,
    rmCache = TRUE,
    path = outputDir))

# run shiny app

if (interactive())
    ScaleTransformApp(dir = outputDir)                     

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plotScaleTransformedChannel}
\alias{plotScaleTransformedChannel}
\title{Plot a flow frame in 1D with explicit user given scale transform}
\usage{
plotScaleTransformedChannel(
  ff,
  channel,
  applyTransform = c("axis scale only", "data"),
  transfoType = c("linear", "logicle"),
  linA,
  linB,
  negDecades,
  width,
  posDecades
)
}
\arguments{
\item{ff}{the \code{flowFrame} to be plotted}

\item{channel}{the name of the channel of which to display the marginal
distribution (i.e. the channel name used as column in the ff expression
matrix).}

\item{applyTransform}{if "data", data are explicitly transformed using
the user provided sclae transformation parameters, before display
if "axis scale only" (default), the data are not transformed,
i.e. only the x axis scale is defined according to
the scale transformation parameters.}

\item{transfoType}{the transformation type, currently only
\code{linear} and \code{logicle}(bi-exponential) are supported.}

\item{linA}{the intercept parameter of the linear transformation.}

\item{linB}{the slope parameter of the linear transformation.}

\item{negDecades}{the number of additional decades on the negative side
for the logicle transformation.}

\item{width}{the width parameter of the logicle transformation.}

\item{posDecades}{the number of positive decades of the logicle
tranformation.}
}
\value{
a ggplot object
}
\description{
This function plots a 1D view, i.e. the marginal distribution
for one specified channel, of the given flow frame, using the specific
user-provided scale transformation parameters.
}
\examples{
# run CytoPipeline object first

outputDir <- base::tempdir()


rawDataDir <-
    system.file("extdata", package = "CytoPipeline")
experimentName <- "OMIP021_PeacoQC"
sampleFiles <- 
    file.path(
        rawDataDir, 
        list.files(rawDataDir, pattern = "Donor"))
jsonDir <- system.file("extdata", package = "CytoPipeline")
jsonPath <- file.path(jsonDir, "pipelineParams.json")

pipL2 <- CytoPipeline(
    jsonPath,
    experimentName = experimentName,
    sampleFiles = sampleFiles)

suppressWarnings(execute(
    pipL2,
    rmCache = TRUE,
    path = outputDir))

ff <- CytoPipeline::getCytoPipelineFlowFrame(
pipL2,
path = outputDir,
whichQueue = "scale transform",
objectName = "flowframe_aggregate_obj"
)

plotScaleTransformedChannel(
    ff,
    channel = "FSC-A",
    transfoType = "linear",
    linA = 0.0002,
    linB = -0.5)

plotScaleTransformedChannel(
    ff,
    channel = "Comp-670/30Violet-A",
    transfoType = "logicle",
    negDecades = 1,
    width = 0.5,
    posDecades = 4
)

plotScaleTransformedChannel(
    ff,
    channel = "CD3",
    applyTransform = "data",
    transfoType = "logicle",
    negDecades = 1,
    width = 0.5,
    posDecades = 4
)

}
